import re


def center(self, node="", node1="", node2="", node3="", radius="",
           **kwargs):
    """Defines a node at the center of curvature of 2 or 3 nodes.

    APDL Command: CENTER

    Parameters
    ----------
    node
        Number to be assigned to the node generated at the center of
        curvature.

    node1, node2, node3
        Three nodes used to calculated the center of curvature, as
        described under RADIUS.

    radius
        Used to control the interpretation of NODE1, NODE2 and NODE3:

        0 - NODE1, NODE2 and NODE3 lie on a circular arc.  The program will calculate the
            center of curvature (and radius) (default).

        ≠0 - NODE1 and NODE2 are the endpoints of an arc, and RADIUS is the radius of
             curvature.  The program will locate the center of curvature on
             the NODE3 side of the NODE1-NODE2 line if RADIUS > 0, and
             opposite to NODE3 if RADIUS < 0.
    """
    command = f"CENTER,{node},{node1},{node2},{node3},{radius}"
    return self.run(command, **kwargs)


def fill(self, node1="", node2="", nfill="", nstrt="", ninc="", itime="",
         inc="", space="", **kwargs):
    """Generates a line of nodes between two existing nodes.

    APDL Command: FILL

    Parameters
    ----------
    node1, node2
        Beginning and ending nodes for fill-in.  NODE1 defaults to next to
        last node specified, NODE2 defaults to last node specified.  If
        NODE1 = P, graphical picking is enabled and all remaining command
        fields are ignored (valid only in the GUI).

    nfill
        Fill NFILL nodes between NODE1 and NODE2 (defaults to
        |NODE2-NODE1|-1).  NFILL must be positive.

    nstrt
        Node number assigned to first filled-in node (defaults to NODE1 +
        NINC).

    ninc
        Add this increment to each of the remaining filled-in node numbers
        (may be positive or negative).  Defaults to the integer result of
        (NODE2-NODE1)/(NFILL + 1), i.e., linear interpolation.  If the
        default evaluates to zero, or if zero is input, NINC is set to 1.

    itime, inc
        Do fill-in operation a total of ITIMEs, incrementing NODE1, NODE2
        and NSTRT by INC each time after the first.  ITIME and INC both
        default to 1.

    space
        Spacing ratio.  Ratio of last division size to first division size.
        If > 1.0, divisions increase.  If < 1.0, divisions decrease.  Ratio
        defaults to 1.0 (uniform spacing).

    Notes
    -----
    Generates a line of nodes (in the active coordinate system) between two
    existing nodes.  The two nodes may have been defined in any coordinate
    system.  Nodal locations and rotation angles are determined by
    interpolation.  Any number of nodes may be filled-in and any node
    number sequence may be assigned.  See the CSCIR command when filling
    across the 180° singularity line in a non-Cartesian system.
    """
    command = f"FILL,{node1},{node2},{nfill},{nstrt},{ninc},{itime},{inc},{space}"
    return self.run(command, **kwargs)


def move(self, node="", kc1="", x1="", y1="", z1="", kc2="", x2="", y2="",
         z2="", **kwargs):
    """Calculates and moves a node to an intersection.

    APDL Command: MOVE

    Parameters
    ----------
    node
        Move this node.  If NODE = P, graphical picking is enabled and all
        remaining command fields are ignored (valid only in the GUI).  A
        component name may also be substituted for NODE.

    kc1
        First coordinate system number.  Defaults to 0 (global Cartesian).

    x1, y1, z1
        Input one or two values defining the location of the node in this
        coordinate system.  Input "U" for unknown value(s) to be calculated
        and input "E" to use an existing coordinate value.  Fields are R1,
        θ1, Z1 for cylindrical, or R1, θ1, Φ1 for spherical or toroidal.

    kc2
        Second coordinate system number.

    x2, y2, z2
        Input two or one value(s) defining the location of the node in this
        coordinate system.  Input "U" for unknown value(s) to be calculated
        and input "E" to use an existing coordinate value.  Fields are R2, 
        θ2, Z2 for cylindrical, or R2,  θ2, Φ2 for spherical or toroidal.

    Notes
    -----
    Calculates and moves a node to an intersection location.  The node may
    have been previously defined (at an approximate location) or left
    undefined (in which case it is internally defined at the SOURCE
    location).  The actual location is calculated from the intersection of
    three surfaces (implied from three coordinate constants in two
    different coordinate systems).  The three (of six) constants easiest to
    define should be used.  The program will calculate the remaining three
    coordinate constants.  All arguments, except KC1, must be input.  Use
    the repeat command [*REPEAT] after the MOVE command to define a line of
    intersection by repeating the move operation on all nodes of the line.

    Surfaces of constant value are implied by some commands by specifying a
    single coordinate value.  Implied surfaces are used with various
    commands [MOVE, KMOVE, NSEL, etc.].  Three surfaces are available with
    each of the four coordinate system types.  Values or X, Y, or Z may be
    constant for the Cartesian coordinate system; values of R,: θ, or Z for
    the cylindrical system; and values of R, θ,: Φ for the spherical and
    toroidal systems.  For example, an X value of 3 represents the Y-Z
    plane (or surface) at X=3.  In addition, the parameters for the
    cylindrical and spherical coordinate systems may be adjusted [CS,
    LOCAL] to form elliptical surfaces.  For surfaces in elliptical
    coordinate systems, a surface of "constant" radius is defined by the
    radius value at the X-axis.  Surfaces of constant value may be located
    in local coordinate systems [LOCAL, CLOCAL, CS, or CSKP] to allow for
    any orientation.

    The intersection calculation is based on an iterative procedure (250
    iterations maximum) and a tolerance of 1.0E-4.  The approximate
    location of a node should be sufficient to determine a unique
    intersection if more than one intersection point is possible.  Tangent
    "intersections" should be avoided.  If an intersection is not found,
    the node is placed at the last iteration location.

    This command is also valid in the /MAP processor.
    """
    command = f"MOVE,{node},{kc1},{x1},{y1},{z1},{kc2},{x2},{y2},{z2}"
    return self.run(command, **kwargs)


def n(self, node="", x="", y="", z="", thxy="", thyz="", thzx="",
      **kwargs) -> int:
    """Define a node.

    APDL Command: N

    Parameters
    ----------
    node
        Node number to be assigned.  A previously defined node of
        the same number will be redefined.  Defaults to the
        maximum node number used +1.

    x, y, z
        Node location in the active coordinate system (R, θ, Z for
        cylindrical, R, θ, Φ for spherical or toroidal).

    thxy
        First rotation about nodal Z (positive X toward Y).

    thyz
        Second rotation about nodal X (positive Y toward Z).

    thzx
        Third rotation about nodal Y (positive Z toward X).

    Returns
    -------
    int
        Node number of the generated node.

    Examples
    --------
    Create a node at ``(0, 1, 1)``

    >>> nnum = mapdl.n("", 0, 1, 1)
    >>> nnum
    1

    Create a node at ``(4, 5, 1)`` with a node ID of 10

    >>> nnum = mapdl.n(10, 4, 5, 1)
    >>> nnum
    10

    Notes
    -----
    Defines a node in the active coordinate system [CSYS].  The
    nodal coordinate system is parallel to the global Cartesian
    system unless rotated.  Rotation angles are in degrees and
    redefine any previous rotation angles.  See the NMODIF, NANG,
    NROTAT, and NORA commands for other rotation options.
    """
    command = f"N,{node},{x},{y},{z},{thxy},{thyz},{thzx}"
    msg = self.run(command, **kwargs)
    if msg:
        res = re.search(r"(NODE\s*)([0-9]+)", msg)
        if res is not None:
            return int(res.group(2))


def naxis(self, action="", val="", **kwargs):
    """Generates nodes for general axisymmetric element sections.

    APDL Command: NAXIS

    Parameters
    ----------
    action
        Specifies one of the following command behaviors:

        GEN - Generates nodes around the axis of an axisymmetric section (default).

        CLEAR - Clears all nodes around the axis of an axisymmetric section.

        EFACET - Specifies the number of facets per edge between nodal planes and integration
                 planes in the circumferential direction to display using
                 PowerGraphics.  This option is only valid with /ESHAPE,1
                 and RSYS,SOLU commands.

    val
        Tolerance value or number of facets per edge:

        TOLER - When Action = GEN, the tolerance to use for merging the generated nodes around
                the axis.

        NUM - When Action = EFACET, the number of facets per element edge for element plots:

        AUTO - Use program-chosen facets per edge (default).

        1 - Use 1 facet per edge (default for elements with 9, 10, 11, or 12 nodal planes).
            Shows nodal and integration planes only.

        2 - Use 2 facets per edge (default for elements with 5, 6, 7, or 8 nodal planes,
            and maximum for elements with 9, 10, 11, or 12 nodal planes).

        3 - Use 3 facets per edge (default for elements with 3 or 4 nodal planes, and
            maximum for elements with 6, 7, or 8 nodal planes).

        4 - Use 4 facets per edge (maximum for elements with 5 nodal planes).

        5 - Use 5 facets per edge (maximum for elements with 4 nodal planes).

        6 - Use 6 facets per edge (maximum for elements with 3 nodal planes).

    Notes
    -----
    The NAXIS command generates or clears the nodes for general
    axisymmetric element sections. The command applies to elements SURF159,
    SOLID272, and SOLID273.

    The generate option (Action = GEN) operates automatically on any
    current-technology axisymmetric element. Any nodes within the tolerance
    value (TOLER) of the axis are merged into a single node. The default
    tolerance is 1.0e-4.

    If you want to change the number of nodes, use the clear option (Action
    = CLEAR) before regenerating the nodes.

    To cause the 3-D element plot to appear more like the actual 3-D model,
    use NAXIS,EFACET,NUM, where NUM > 1. In this case, the coordinate
    system specified for displaying element and nodal results (RSYS) must
    be solution (RSYS,SOLU); otherwise, ANSYS resets NUM to 1.
    """
    command = f"NAXIS,{action},{val}"
    return self.run(command, **kwargs)


def nang(self, node="", x1="", x2="", x3="", y1="", y2="", y3="", z1="",
         z2="", z3="", **kwargs):
    """Rotates a nodal coordinate system by direction cosines.

    APDL Command: NANG

    Parameters
    ----------
    node
        Rotate coordinate system of this node.

    x1, x2, x3
        Global X, Y, Z components of a unit vector in new nodal X
        direction.

    y1, y2, y3
        Global X, Y, Z components of a unit vector in new nodal Y
        direction.

    z1, z2, z3
        Global X, Y, Z components of a unit vector in new nodal Z
        direction.

    Notes
    -----
    Rotates a nodal coordinate system to the orientation specified by the
    X, Y and Z direction cosines.  Existing rotation specifications on the
    node are redefined.  If only two of the three unit vectors are
    specified, the third is defined according to the right hand rule.  It
    is the responsibility of the user to ensure that input direction
    cosines are orthogonal in a right-handed system.

    See the NMODIF, NROTAT, and NORA commands for other rotation options.
    """
    command = f"NANG,{node},{x1},{x2},{x3},{y1},{y2},{y3},{z1},{z2},{z3}"
    return self.run(command, **kwargs)


def ndele(self, node1="", node2="", ninc="", **kwargs):
    """Deletes nodes.

    APDL Command: NDELE

    Parameters
    ----------
    node1, node2, ninc
        Delete nodes from NODE1 to NODE2 (defaults to NODE1) in steps of
        NINC (defaults to 1).  If NODE1 = ALL, NODE2 and NINC are ignored
        and all selected nodes [NSEL] are deleted.  If NODE1 = P, graphical
        picking is enabled and all remaining command fields are ignored
        (valid only in the GUI).  A component name may also be substituted
        for NODE1.

    Notes
    -----
    Deletes selected nodes that are not connected to elements.  Nodes may
    also be redefined instead of deleted, if desired.  Boundary conditions
    (displacements, forces, etc.) as well as any coupling or constraint
    equations containing the deleted nodes are also deleted.

    This command is also valid in the /MAP processor.
    """
    command = f"NDELE,{node1},{node2},{ninc}"
    return self.run(command, **kwargs)


def ndist(self, nd1="", nd2="", **kwargs):
    """Calculates and lists the distance between two nodes.

    APDL Command: NDIST

    Parameters
    ----------
    nd1
        First node in distance calculation.  If ND1 = P, graphical picking
        is enabled and all remaining command fields are ignored (valid only
        in the GUI).

    nd2
        Second node in distance calculation.

    Notes
    -----
    NDIST lists the distance between nodes ND1 and ND2, as well as the
    current coordinate system offsets from ND1 to ND2, where the X, Y, and
    Z locations of ND1 are subtracted from the X, Y, and Z locations of ND2
    (respectively) to determine the offsets.  NDIST is valid in any
    coordinate system except toroidal [CSYS,3].

    NDIST returns a variable, called "_RETURN," which contains the distance
    value. You can use this value for various purposes, such as the
    calculation of distributed loads. In interactive mode, you can access
    this command by using the Model Query Picker (Utility Menu> List>
    Picked Entities), where you can also access automatic annotation
    functions and display the value on your model.

    This command is valid in any processor.
    """
    command = f"NDIST,{nd1},{nd2}"
    return self.run(command, **kwargs)


def ngen(self, itime="", inc="", node1="", node2="", ninc="", dx="", dy="",
         dz="", space="", **kwargs):
    """Generates additional nodes from a pattern of nodes.

    APDL Command: NGEN

    Parameters
    ----------
    itime, inc
        Do this generation operation a total of ITIME times, incrementing
        all nodes in the given pattern by INC each time after the first.
        ITIME must be > 1 for generation to occur.

    node1, node2, ninc
        Generate nodes from the pattern of nodes beginning with NODE1 to
        NODE2 (defaults to NODE1) in steps of NINC (defaults to 1).  If
        NODE1 = ALL, NODE2 and NINC are ignored and the pattern is all
        selected nodes [NSEL].  If NODE1 = P, graphical picking is enabled
        and all remaining command fields are ignored (valid only in the
        GUI).  A component name may also be substituted for NODE1 (NODE2
        and NINC are ignored).

    dx, dy, dz
        Node location increments in the active coordinate system (DR, Dθ,
        DZ for cylindrical, DR, Dθ, DΦ for spherical or toroidal).

    space
        Spacing ratio.  Ratio of last division size to first division size.
        If > 1.0, divisions increase.  If < 1.0, divisions decrease.  Ratio
        defaults to 1.0 (uniform spacing).

    Notes
    -----
    Generates additional nodes from a given node pattern.  Generation is
    done in the active coordinate system.  Nodes in the pattern may have
    been generated in any coordinate system.

    This command is also valid in the /MAP processor.
    """
    command = f"NGEN,{itime},{inc},{node1},{node2},{ninc},{dx},{dy},{dz},{space}"
    return self.run(command, **kwargs)


def nkpt(self, node="", npt="", **kwargs):
    """Defines a node at an existing keypoint location.

    APDL Command: NKPT

    Parameters
    ----------
    node
        Arbitrary reference number for node.  If zero or blank, defaults to
        the highest node number +1 [NUMSTR].

    npt
        Keypoint number defining global X, Y, Z  location.  If NPT = All,
        then a node will be placed at each selected keypoint.  If NPT = P,
        graphical picking is enabled and all remaining command fields are
        ignored (valid only in the GUI).  A component name may also be
        substituted for NPT.
    """
    command = f"NKPT,{node},{npt}"
    return self.run(command, **kwargs)


def nlist(self, node1="", node2="", ninc="", lcoord="", sort1="", sort2="",
          sort3="", kinternal="", **kwargs):
    """Lists nodes.

    APDL Command: NLIST

    Parameters
    ----------
    node1, node2, ninc
        List nodes from NODE1 to NODE2 (defaults to NODE1) in steps of NINC
        (defaults to 1).  If NODE1 = ALL (default), NODE2 and NINC are
        ignored and all selected nodes [NSEL] are listed.  If NODE1 = P,
        graphical picking is enabled and all remaining command fields are
        ignored (valid only in the GUI).  A component name may also be
        substituted for NODE1 (NODE2 and NINC are ignored).

    lcoord
        Coordinate listing key:

        (blank) - List all nodal information

        COORD - Suppress all but the XYZ coordinates (shown to a higher degree of accuracy than
                when displayed with all information).

    sort1
        First item on which to sort.  Valid item names are NODE, X, Y, Z,
        THXY, THYZ, THXZ

    sort2, sort3
        Second and third items on which to sort.  Valid item names are the
        same as for SORT1.

    kinternal
        Internal nodes listing key:

        (blank) - List only external nodes.

        INTERNAL - List all nodes, including internal nodes.

    Notes
    -----
    Lists nodes in the active display coordinate system [DSYS].  Nodal
    coordinate rotation angles are also listed (relative to the global
    Cartesian coordinate system).

    Node listing can be in a sorted order (ascending).  SORT2, for example,
    will be carried out on nodes having equal values of SORT1.

    This command is valid in any processor.
    """
    command = f"NLIST,{node1},{node2},{ninc},{lcoord},{sort1},{sort2},{sort3},{kinternal}"
    return self.run(command, **kwargs)


def nmodif(self, node="", x="", y="", z="", thxy="", thyz="", thzx="",
           **kwargs):
    """Modifies an existing node.

    APDL Command: NMODIF

    Parameters
    ----------
    node
        Modify coordinates of this node.  If ALL, modify coordinates of all
        selected nodes [NSEL].  If NODE = P, graphical picking is enabled
        and all remaining command fields are ignored (valid only in the
        GUI).  A component name may also be substituted for NODE.

    x, y, z
        Replace the previous coordinate values assigned to this node with
        these corresponding coordinate values.  Values are interpreted in
        the active coordinate system (R, θ, Z for cylindrical; R, θ, Φ for
        spherical or toroidal).  Leaving any of these fields blank retains
        the previous value(s).

    thxy
        First rotation of nodal coordinate system about nodal Z (positive X
        toward Y). Leaving this field blank retains the previous value.

    thyz
        Second rotation of nodal coordinate system about nodal X (positive
        Y toward Z). Leaving this field blank retains the previous value.

    thzx
        Third rotation of nodal coordinate system about nodal Y (positive Z
        toward X). Leaving this field blank retains the previous value.

    Notes
    -----
    Modifies an existing node.  Nodal coordinate system rotation angles are
    in degrees and redefine any existing rotation angles.  Nodes can also
    be redefined with the N command.

    See the NROTAT, NANG, and NORA commands for other rotation options.

    This command is also valid in the /MAP processor.
    """
    command = f"NMODIF,{node},{x},{y},{z},{thxy},{thyz},{thzx}"
    return self.run(command, **kwargs)


def nora(self, area="", ndir="", **kwargs):
    """Rotates nodal coordinate systems to surface normal

    APDL Command: NORA

    Parameters
    ----------
    area
        The area number containing the nodes to be rotated to their
        normals. If ALL, applies to all selected areas (see the ASEL
        command). If AREA = P, graphical picking is enabled.

    ndir
        Direction of the normal. If NDIR = -1, the nodal coordinate system
        is rotated in the opposite direction of the surface normal. The
        default is the same direction as the surface normal.

    Notes
    -----
    The NORA command rotates the X-axis of the nodal coordinate system to
    the surface normal. The rotated nodal coordinate systems may be
    displayed through the /PSYMB command. In case multiple areas are
    selected, there could be conflicts at the boundaries. If a node belongs
    to two areas that have a different normal, its nodal coordinate system
    will be rotated to the area normal with the lowest number. You can use
    the AREVERSE and ANORM commands to rotate the surface normals in the
    appropriate direction. Keep the following in mind when using the NORA
    command:

    If the nodal coordinate system is parallel to the global Cartesian
    system, it is not displayed through the /PSYMB command.

    Previously specified rotation on the selected nodes are overridden.
    """
    command = f"NORA,{area},{ndir}"
    return self.run(command, **kwargs)


def norl(self, line="", area="", ndir="", **kwargs):
    """APDL Command: NORL

    Rotates nodal coordinate systems perpendicular to line normal

    Parameters
    ----------
    line
        Line number containing the nodes to be rotated. If ALL, applies to
        all selected lines (see the LSEL command). If LINE = P, graphical
        picking is enabled.

    area
        The area number containing the selected lines. The normal of the
        line(s) selected is supposed to lie on this area. Defaults to the
        lowest numbered selected area containing the line number.

    ndir
        Direction of the normal. If NDIR = -1, the nodal coordinate system
        is rotated in the opposite direction of the line normal. The
        default is the same direction as the surface normal.

    Notes
    -----
    The NORL command rotates the X-axis of the nodal coordinate
    perpendicular to the line normal. The rotated nodal coordinate systems
    may be displayed through the /PSYMB command. In case multiple lines are
    selected, there could be conflicts at the boundaries. If a node belongs
    to two lines that have a different normal, its nodal coordinate system
    will be rotated to the line normal with the lowest number. Keep the
    following in mind when using the NORL command:

    If the nodal coordinate system is parallel to the global Cartesian
    system, it is not displayed through the /PSYMB command.

    Previously specified rotation on the selected nodes are overridden.
    """
    command = f"NORL,{line},{area},{ndir}"
    return self.run(command, **kwargs)


def nplot(self, knum="", **kwargs):
    """Displays nodes.

    APDL Command: NPLOT

    Parameters
    ----------
    knum
        Node number key:

        0 - No node numbers on display.

        1 - Include node numbers on display.  See also /PNUM command.

    Notes
    -----
    Produces a node display.  Only selected nodes [NSEL] are displayed.
    Elements need not be defined.  See the DSYS command for display
    coordinate system.

    This command is valid in any processor.
    """
    command = f"NPLOT,{knum}"
    return self.run(command, **kwargs)


def nread(self, fname="", ext="", **kwargs):
    """Reads nodes from a file.

    APDL Command: NREAD

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

    ext
        Filename extension (eight-character maximum).

    Notes
    -----
    The read operation is not necessary in a standard ANSYS run but is
    provided as a convenience to users wanting to read a coded node file,
    such as from another mesh generator or from a CAD/CAM program.  Data
    should be formatted as produced with the NWRITE command. Only nodes
    that are within the node range specified with the NRRANG command are
    read from the file.  Duplicate nodes already in the database will be
    overwritten.  The file is rewound before and after reading.  Reading
    continues until the end of the file.
    """
    command = f"NREAD,{fname},{ext}"
    return self.run(command, **kwargs)


def nrotat(self, node1="", node2="", ninc="", **kwargs):
    """Rotates nodal coordinate systems into the active system.

    APDL Command: NROTAT

    Parameters
    ----------
    node1, node2, ninc
        Rotate nodes from NODE1 to NODE2 (defaults to NODE1) in steps of
        NINC (defaults to 1).  If NODE1 = ALL, NODE2 and NINC are ignored
        and all selected nodes [NSEL] are rotated.  If NODE1 = P, graphical
        picking is enabled and all remaining command fields are ignored
        (valid only in the GUI).  A component name may also be substituted
        for NODE1 (NODE2 and NINC are ignored).

    Notes
    -----
    Rotates nodal coordinate systems into the active coordinate system.
    Nodal coordinate systems may be automatically rotated into the active
    (global or local) coordinate system as follows:  Rotations in Cartesian
    systems will have nodal x directions rotated parallel to the Cartesian
    X direction.  Rotations in cylindrical, spherical or toroidal systems
    will have the nodal x directions rotated parallel to the R direction.
    Nodes at (or near) a zero radius location should not be rotated.  Nodal
    coordinate directions may be displayed [/PSYMB].  Nodal forces and
    constraints will also appear rotated when displayed if the nodal
    coordinate system is rotated.

    ANSYS LS-DYNA (explicit dynamics) does not support the NROTAT command.
    If you have rotated nodes in the implicit phase of an implicit-to-
    explicit sequential solution, you must rotate the nodes back to the
    global Cartesian direction before switching from implicit to explicit
    elements (ETCHG,ITE). Use the EDNROT command in the explicit run to
    maintain the same displacement constraints as were used on rotated
    nodes in the implicit run.

    Note:: : When the  nodal coordinate systems are defined, they remain
    parallel to the global Cartesian system unless subsequently rotated.

    Previously specified rotations on the specified nodes are overridden.

    See the NMODIF, NANG, and NORA commands for other rotation options.
    """
    command = f"NROTAT,{node1},{node2},{ninc}"
    return self.run(command, **kwargs)


def nrrang(self, nmin="", nmax="", ninc="", **kwargs):
    """Specifies the range of nodes to be read from the node file.

    APDL Command: NRRANG

    Parameters
    ----------
    nmin, nmax, ninc
        Node range is defined from NMIN (defaults to 1) to NMAX (defaults
        to 99999999) in steps of NINC (defaults to 1).

    Notes
    -----
    Defines the range of nodes to be read [NREAD] from the node file.  Also
    implies an element range since only elements fully attached to these
    nodes will be read from the element file.
    """
    command = f"NRRANG,{nmin},{nmax},{ninc}"
    return self.run(command, **kwargs)


def nscale(self, inc="", node1="", node2="", ninc="", rx="", ry="", rz="",
           **kwargs):
    """Generates a scaled set of nodes from a pattern of nodes.

    APDL Command: NSCALE

    Parameters
    ----------
    inc
        Do this scaling operation one time, incrementing all nodes in the
        given pattern by INC.  If INC = 0, nodes will be redefined at the
        scaled locations.

    node1, node2, ninc
        Scale nodes from pattern of nodes beginning with NODE1 to NODE2
        (defaults to NODE1) in steps of NINC (defaults to 1).  If  NODE1 =
        ALL, NODE2 and NINC are ignored and pattern is all selected nodes
        [NSEL].  If NODE1 = P, graphical picking is enabled and all
        remaining command fields are ignored (valid only in the GUI).  A
        component name may also be substituted for NODE1 (NODE2 and NINC
        are ignored).

    rx, ry, rz
        Scale factor ratios.  Scaling is relative to the origin of the
        active coordinate system (RR, Rθ, RZ for cylindrical, RR, Rθ, RΦ
        for spherical or toroidal).  If absolute value of ratio > 1.0,
        pattern is enlarged.  If < 1.0, pattern is reduced.  Ratios default
        to 1.0 (each).

    Notes
    -----
    Generates a scaled pattern of nodes from a given node pattern.  Scaling
    is done in the active coordinate system.  Nodes in the pattern may have
    been generated in any coordinate system.

    This command is also valid in the /MAP processor.
    """
    command = f"NSCALE,{inc},{node1},{node2},{ninc},{rx},{ry},{rz}"
    return self.run(command, **kwargs)


def nsmooth(self, npass="", **kwargs):
    """Smooths selected nodes among selected elements.

    APDL Command: NSMOOTH

    Parameters
    ----------
    npass
        Number of smoothing passes. Defaults to 3.

    Notes
    -----
    Repositions each selected node at the average position of its immediate
    neighbors on the selected elements. The node positions converge after
    some number of smoothing passes. For some initial conditions, NPASS may
    need to be much larger than 3. If the boundary of a mesh is to be
    undisturbed (usually desirable), the boundary nodes should be
    unselected before issuing NSMOOTH.
    """
    command = f"NSMOOTH,{npass}"
    return self.run(command, **kwargs)


def nsym(self, ncomp="", inc="", node1="", node2="", ninc="", **kwargs):
    """Generates a reflected set of nodes.

    APDL Command: NSYM

    Parameters
    ----------
    ncomp
        Symmetry key:

        X - X (or R) symmetry (default).

        Y - Y (or θ) symmetry.

        Z - Z (or Φ) symmetry.

    inc
        Increment all nodes in the given pattern by INC to form the
        reflected node pattern.

    node1, node2, ninc
        Reflect nodes from pattern beginning with NODE1 to NODE2 (defaults
        to NODE1) in steps of NINC (defaults to 1).  If  NODE1 = ALL, NODE2
        and NINC are ignored and pattern is all selected nodes [NSEL].  If
        NODE1 = P, graphical picking is enabled and all remaining command
        fields are ignored (valid only in the GUI).  A component name may
        also be substituted for NODE1 (NODE2 and NINC are ignored).

    Notes
    -----
    Generates nodes from a given node pattern by a symmetry reflection.
    Reflection is done in the active coordinate system by changing a
    particular coordinate sign.  Nodes in the pattern may have been
    generated in any coordinate system.   Nodal rotation angles are not
    reflected.

    Symmetry reflection may be used with any node pattern, in any
    coordinate system, as many times as desired.  Reflection is
    accomplished by a coordinate sign change (in the active coordinate
    system).  For example, an X-reflection in a Cartesian coordinate system
    generates additional nodes from a given pattern, with a node increment
    added to each node number, and an X coordinate sign change.  An
    R-reflection in a cylindrical coordinate system gives a reflected
    "radial" location by changing the "equivalent" Cartesian (i.e., the
    Cartesian system with the same origin as the active cylindrical system)
    X and Y coordinate signs.  An R-reflection in a spherical coordinate
    system gives a reflected "radial" location by changing the equivalent
    Cartesian X, Y, and Z coordinate location signs.  Nodal coordinate
    system rotation angles are not reflected.
    """
    command = f"NSYM,{ncomp},{inc},{node1},{node2},{ninc}"
    return self.run(command, **kwargs)


def nwrite(self, fname="", ext="", kappnd="", **kwargs):
    """Writes nodes to a file.

    APDL Command: NWRITE

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum,
        including the characters needed for the directory path).
        An unspecified directory path defaults to the working
        directory; in this case, you can use all 248 characters
        for the file name.

    ext
        Filename extension (eight-character maximum).

    kappnd
        Append key:

        0 - Rewind file before the write operation.

        1 - Append data to the end of the existing file.

    Notes
    -----
    Writes selected nodes [NSEL] to a file.  The write operation
    is not necessary in a standard ANSYS run but is provided as a
    convenience to users wanting a coded node file.  Data are
    written in a coded format.  The format used is (I8, 6G20.13)
    to write out NODE,X,Y,Z,THXY,THYZ,THZX.  If the last number is
    zero (i.e., THZX = 0), or the last set of numbers are zero,
    they are not written but are left blank. Therefore, you must
    use a formatted read to process this file.  Coordinate values
    are in the global Cartesian system.
    """
    return self.run(f"NWRITE,{fname},{ext},,{kappnd}", **kwargs)


def quad(self, node1="", nintr="", node2="", nfill="", nstrt="", ninc="",
         pkfac="", **kwargs):
    """Generates a quadratic line of nodes from three nodes.

    APDL Command: QUAD

    Parameters
    ----------
    node1
        Begin fill-in from this node location.  If NODE1 = P, graphical
        picking is enabled and all remaining command fields are ignored
        (valid only in the GUI).

    nintr
        Intermediate or guiding node.  Quadratic curve will pass through
        this location.  NINTR may have any node number and any location.
        If the quadratic line also generates a node with number NINTR, the
        generated location overrides the previous NINTR location.

    node2
        End quadratic fill-in at this node location.

    nfill
        Fill-in NFILL nodes between NODE1 and NODE2 (defaults to
        |NODE2-NODE1|-1).  NFILL must be positive.

    nstrt
        Node number assigned to first filled-in node (defaults to NODE1 +
        NINC).

    ninc
        Add this increment to each of the remaining filled-in node numbers
        (may be positive or negative).  Defaults to (NODE2-NODE1)/(NFILL +
        1), i.e., linear interpolation.

    pkfac
        Peak location factor.  If PKFAC=0.5, the peak of the quadratic
        shape occurs at the NINTR location.  If 0.0 < PKFAC < 0.5, the peak
        occurs to the NODE2 side of the NINTR location.  If 0.5 < PKFAC <
        1.0, the peak occurs to the NODE1 side of the NINTR location.
        Defaults to 0.5.

    Notes
    -----
    Generates a quadratic line of nodes (in the active coordinate system)
    from three nodes.  The three nodes determine the plane of the curve and
    may have been defined in any coordinate system.  Any number of nodes
    may be filled-in and any node number sequence may be assigned.

    The quadratic line feature uses three nodes (NODE1,NINTR,NODE2) to
    determine the plane of the curve.  The curve passes through the three
    points, beginning from NODE1, through the intermediate (or guiding)
    point NINTR, and toward NODE2.

    Generated nodes are also quadratically spaced.  If the guiding node
    number is within the set being generated, it will be relocated
    according to the quadratic spacing.

    The peak location factor is used to determine how the quadratic fits
    through the three points.  Various nodal progressions can be obtained
    by different combinations of PKFAC and the guiding node location.  If
    the guiding node is at mid-length between NODE1 and NODE2, 0.293:
    PKFAC< 0.707 will ensure that all generated nodes fall within the
    NODE1,NODE2 bounds.  In the limit, as PKFAC approaches 0.0, the peak
    approaches the line through NODE1 and NINTR at an infinite distance
    from NODE1.  The QUAD command generates quadratic lines of nodes, which
    in turn may be used as a base line for generating irregular surfaces of
    nodes (by repeating [*REPEAT], generating [NGEN, NSCALE], etc.).
    Irregular surfaces may also be generated with the meshing commands.
    """
    command = f"QUAD,{node1},{nintr},{node2},{nfill},{nstrt},{ninc},{pkfac}"
    return self.run(command, **kwargs)


def transfer(self, kcnto="", inc="", node1="", node2="", ninc="",
             **kwargs):
    """Transfers a pattern of nodes to another coordinate system.

    APDL Command: TRANSFER

    Parameters
    ----------
    kcnto
        Reference number of coordinate system where the pattern is to be
        transferred.  Transfer occurs from the active coordinate system.

    inc
        Increment all nodes in the given pattern by INC to form the
        transferred node pattern.

    node1, node2, ninc
        Transfer nodes from pattern beginning with NODE1 to NODE2 (defaults
        to NODE1) in steps of NINC (defaults to 1).  If NODE1 = ALL, NODE2
        and NINC are ignored and the pattern is all selected nodes [NSEL].
        If NODE1 = P, graphical picking is enabled and all remaining
        command fields are ignored (valid only in the GUI).  A component
        may be substituted for NODE1 (NODE2 and NINC are ignored).

    Notes
    -----
    Transfers a pattern of nodes from one coordinate system to another.
    Coordinate systems may be translated and rotated relative to each
    other.  Initial pattern may be generated in any coordinate system.
    Coordinate values are interpreted in the active coordinate system and
    are transferred directly.

    A model generated in one coordinate system may be transferred to
    another coordinate system.  The user may define several coordinate
    systems (translated and rotated from each other), generate a model in
    one coordinate system, and then repeatedly transfer the model to other
    coordinate systems.  The model may be generated in any type of
    coordinate system (Cartesian, cylindrical, etc.) and transferred to any
    other type of coordinate system.  Coordinate values (X, Y, Z, or R,: θ,
    Z, or etc.) of the model being transferred are interpreted in the
    active coordinate system type, regardless of how they were generated.
    Values are transferred directly and are interpreted according to the
    type of coordinate system being transferred to.  For example,
    transferring from a Cartesian coordinate system to a cylindrical
    coordinate system (not recommended) would cause X = 2.0 and Y = 3.0
    values to be directly interpreted as R = 2.0 and θ = 3.0 values,
    respectively.

    This command is also valid in the /MAP processor.
    """
    command = f"TRANSFER,{kcnto},{inc},{node1},{node2},{ninc}"
    return self.run(command, **kwargs)



def sf(self, nlist="", lab="", value="", value2="", **kwargs):
    """Specifies surface loads on nodes.

    APDL Command: SF

    Parameters
    ----------
    nlist
        Nodes defining the surface upon which the load is to be applied.
        Use the label ALL or P, or a component name.  If ALL, all selected
        nodes [NSEL] are used (default).  If P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).

    lab
        Valid surface load label. Load labels are listed under "Surface
        Loads" in the input table for each element type in the Element
        Reference.

    value
        Surface load value or table name reference for specifying tabular
        boundary conditions.

    value2
        Second surface load value (if any).

    Notes
    -----
    Individual nodes may not be entered for this command.  The node list is
    to identify a surface and the Nlist field must contain a sufficient
    number of nodes to define an element surface.  The loads are internally
    stored on element faces defined by the specified nodes.  All nodes on
    an element face (including midside nodes, if any) must be specified for
    the face to be used, and the element must be selected.

    If all nodes defining a face are shared by an adjacent face of another
    selected element, the face is not free and will not have a load
    applied.  If more than one element can share the same nodes (for
    example, a surface element attached to a solid element), select the
    desired element type before issuing the SF command. The SF command
    applies only to area and volume elements.

    For shell elements, if the specified nodes include face one (which is
    usually the bottom face) along with other faces (such as edges), only
    face one is used.  Where faces cannot be uniquely determined from the
    nodes, or where the face does not fully describe the load application,
    use the SFE command.  A load key of 1 (which is typically the first
    loading condition on the first face) is used if the face determination
    is not unique.  A uniform load value is applied over the element face.
    """
    command = f"SF,{nlist},{lab},{value},{value2}"
    return self.run(command, **kwargs)


def sfbeam(self, elem="", lkey="", lab="", vali="", valj="", val2i="",
           val2j="", ioffst="", joffst="", lenrat="", **kwargs):
    """Specifies surface loads on beam and pipe elements.

    APDL Command: SFBEAM

    Parameters
    ----------
    elem
        Element to which surface load is applied.  If ALL, apply load to
        all selected beam elements (ESEL).  If Elem = P, graphical picking
        is enabled and all remaining command fields are ignored (valid only
        in the GUI).  A component name may be substituted in Elem.

    lkey
        Load key associated with surface load (defaults to 1).  Load keys
        (1, 2, 3, etc.) are listed under "Surface Loads" in the input table
        for each element type in the Element Reference.  For beam and some
        pipe elements, the load key defines the load orientation.

    lab
        Valid surface load label.  Load labels are listed under "Surface
        Loads" in the input table for each element type in the Element
        Reference.  Structural labels:  PRES (pressure).

    vali, valj
        Surface load values at nodes I and J.  If VALJ is blank, it
        defaults to VALI.  If VALJ is zero, a zero is used.

    val2i, val2j
        Second surface load values at nodes I and J.  Currently not used.

    ioffst, joffst
        Offset distance from node I (toward node J) where VALI is applied,
        and offset distance from node J (toward node I) where VALJ is
        applied, respectively.

    lenrat
        Offset distance flag:

        0  - Offset is in terms of length units (default).

        1  - Offset is in terms of a length ratio (0.0 to 1.0).

    Notes
    -----
    Specifies surface loads on the selected beam elements. Distributed
    loads are applied on a force-per-length basis (that is, the width of
    the underlying element is not considered). To list and delete surface
    loads applied with this command, use the SFELIST and SFEDELE commands,
    respectively.

    If no offset values (IOFFSET and JOFFSET) are specified, the load is
    applied over the full element length. Values may also be input as
    length fractions, depending on the LENRAT setting. For example,
    assuming a line length of 5.0, an IOFFST of 2.0 with LENRAT = 0 or an
    IOFFST of 0.4 with LENRAT = 1 represent the same point.  If JOFFST =
    -1, VALI is assumed to be a point load at the location specified via
    IOFFST, and VALJ is ignored. (IOFFSET cannot be equal to -1.) The
    offset values are stepped even if you issue a KBC,0 command.

    Offsets are only available for element types BEAM188 and PIPE288 if
    using the cubic shape function (KEYOPT(3) = 3) for those element types.

    To accumulate (add) surface loads applied with this command, use the
    SFCUM,,ADD command. Use the same offset values used on the previous
    SFBEAM command (for a given element face); otherwise, the loads do not
    accumulate. If no offsets are specified, the command applies the
    previous offset values.

    This command is also valid in PREP7.
    """
    command = f"SFBEAM,{elem},{lkey},{lab},{vali},{valj},{val2i},{val2j},{ioffst},{joffst},{lenrat}"
    return self.run(command, **kwargs)


def sfcum(self, lab="", oper="", fact="", fact2="", **kwargs):
    """Specifies that surface loads are to be accumulated.

    APDL Command: SFCUM

    Parameters
    ----------
    lab
        Valid surface load label.  If ALL, use all appropriate labels.

    oper
        Accumulation key:

        REPL - Subsequent values replace the previous values (default).

        ADD - Subsequent values are added to the previous values.

        IGNO - Subsequent values are ignored.

    fact
        Scale factor for the first surface load value. A (blank) or '0'
        entry defaults to 1.0.

    fact2
        Scale factor for the second surface load value. A (blank) or '0'
        entry defaults to 1.0.

    Notes
    -----
    Allows repeated surface loads (pressure, convection, etc.) to be
    replaced, added, or ignored.  Surface loads are applied with the SF,
    SFE, and SFBEAM commands.  Issue the SFELIST command to list the
    surface loads.  The operations occur when the next surface load
    specifications are defined.  For example, issuing the SF command with a
    pressure value of 25 after a previous SF command with a pressure value
    of 20 causes the current value of that pressure to be 45 with the add
    operation, 25 with the replace operation, or 20 with the ignore
    operation.  All new pressures applied with SF after the ignore
    operation will be ignored, even if no current pressure exists on that
    surface.

    Scale factors are also available to multiply the next value before the
    add or replace operation.  A scale factor of 2.0 with the previous
    "add" example results in a pressure of 70.  Scale factors are applied
    even if no previous values exist.  Issue SFCUM,STAT to show the current
    label, operation, and scale factors.  Solid model boundary conditions
    are not affected by this command, but boundary conditions on the FE
    model are affected.

    Note:: : The FE boundary conditions may still be overwritten by
    existing solid model boundary conditions if a subsequent boundary
    condition transfer occurs.

    SFCUM does not work for tabular boundary conditions.

    This command is also valid in PREP7.
    """
    command = f"SFCUM,{lab},{oper},{fact},{fact2}"
    return self.run(command, **kwargs)


def sfdele(self, nlist="", lab="", **kwargs):
    """Deletes surface loads.

    APDL Command: SFDELE

    Parameters
    ----------
    nlist
        Label defining where to find the list of nodes:

        ALL - Use all selected nodes [NSEL].  If P, use graphical picking in GUI.  A
              component label may be substituted for Nlist.

    lab
        Valid surface load label.  If ALL, use all appropriate labels.  See
        the SF command for labels.

    Notes
    -----
    Deletes surface loads as applied with the SF command.  Loads are
    deleted only for the specified nodes on external faces of selected area
    and volume elements.  For shell elements, if the specified nodes
    include face one (which is usually the bottom face) along with other
    faces (such as edges), only the loads on face one will be deleted.  The
    element faces are determined from the list of selected nodes as
    described for the SF command.  See the SFEDELE command for deleting
    loads explicitly by element faces.

    This command is also valid in PREP7.
    """
    command = f"SFDELE,{nlist},{lab}"
    return self.run(command, **kwargs)


def sfe(self, elem="", lkey="", lab="", kval="", val1="", val2="", val3="",
        val4="", **kwargs):
    """Specifies surface loads on elements.

    APDL Command: SFE

    Parameters
    ----------
    elem
        Element to which surface load applies.  If ALL, apply load to all
        selected elements [ESEL].  If Elem = P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).  A component name may be substituted for Elem.

    lkey
        Load key or face number associated with surface load (defaults to
        1).  Load keys (1,2,3, etc.) are listed under "Surface Loads" in
        the input data table for each element type in the Element
        Reference.

    lab
        Valid surface load label.  Load labels are listed under "Surface
        Loads" in the input table for each element type in the Element
        Reference.

    kval
        Value key.  If Lab = PRES:

        0 or 1 - VAL1 through VAL4 are used as real components of pressures.

        2 - VAL1 through VAL4 are used as imaginary components of pressures.

    val1
        First surface load value (typically at the first node of the face)
        or the name of a table for specifying tabular boundary conditions.
        Face nodes are listed in the order given for "Surface Loads" in the
        input data table for each element type in the Element Reference.
        For example, for SOLID185, the item 1-JILK associates LKEY = 1
        (face 1) with nodes J, I, L, and K.  Surface load value VAL1 then
        applies to node J of face 1.  To specify a table, enclose the table
        name in percent signs (%), e.g., %tabname%.  Use the *DIM command
        to define a table.  Only one table can be specified, and it must be
        specified in the VAL1 position; tables specified in the VAL2, VAL3,
        or VAL4 positions will be ignored. VAL2 applies to node I, etc.
    """
    command = f"SFE,{elem},{lkey},{lab},{kval},{val1},{val2},{val3},{val4}"
    return self.run(command, **kwargs)


def sfedele(self, elem="", lkey="", lab="", **kwargs):
    """Deletes surface loads from elements.

    APDL Command: SFEDELE

    Parameters
    ----------
    elem
        Element to which surface load deletion applies.  If ALL, delete
        load from all selected elements [ESEL].  If ELEM = P, graphical
        picking is enabled and all remaining command fields are ignored
        (valid only in the GUI).  A component name may be substituted for
        ELEM.

    lkey
        Load key associated with surface load (defaults to 1).  If ALL,
        delete surface loads for all load keys.

    lab
        Valid surface load label.  If ALL, use all appropriate labels.  See
        the SFE command for labels.

    Notes
    -----
    Deletes surface loads from selected elements.  See the SFDELE command
    for an alternate surface load deletion capability based upon selected
    nodes.

    This command is also valid in PREP7.
    """
    command = f"SFEDELE,{elem},{lkey},{lab}"
    return self.run(command, **kwargs)


def sfelist(self, elem="", lab="", **kwargs):
    """Lists the surface loads for elements.

    APDL Command: SFELIST

    Parameters
    ----------
    elem
        Element at which surface load is to be listed.  If ALL (or blank),
        list loads for all selected elements [ESEL].  If ELEM = P,
        graphical picking is enabled and all remaining command fields are
        ignored (valid only in the GUI).  A component name may be
        substituted for ELEM.

    lab
        Valid surface load label.  If ALL (or blank), use all appropriate
        labels.

    Notes
    -----
    The surface loads listed correspond to the current database values. The
    database is not updated for surface loads in POST1. Surface loads
    specified in tabular form, however, do list their values corresponding
    to the current results set in POST1.

    For SURF151 or SURF152 elements with an extra node for radiation and/or
    convection calculations (KEYOPT(5) = 1), the bulk temperature listed is
    the temperature of the extra node. If the thermal solution does not
    converge, the extra node temperature is not available for listing.

    Film effectiveness and free stream temperatures specified by the SFE
    command (Lab = CONV) can only be listed by this command. The command
    lists film coefficients and bulk temperatures first and then film
    effectiveness and free stream temperatures below those values.

    Distributed ANSYS Restriction: In Distributed ANSYS within the SOLUTION
    processor, SFELIST support is not available for elements SURF151 and
    SURF152 when surface loading is applied via extra nodes (KEYOPT(5 > 0).
    If the command is issued under these circumstances, the resulting
    surface loads shown are not reliable.

    This command is valid in any processor.
    """
    command = f"SFELIST,{elem},{lab}"
    return self.run(command, **kwargs)


def sffun(self, lab="", par="", par2="", **kwargs):
    """Specifies a varying surface load.

    APDL Command: SFFUN

    Parameters
    ----------
    lab
        Valid surface load label.  Load labels are listed under "Surface
        Loads" in the input table for each element type in the Element
        Reference.  Issue SFFUN,STATUS to list current command settings.

    par
        Parameter containing list of surface load values.  If Lab = CONV,
        values are typically the film coefficients and Par2 values (below)
        are typically the bulk temperatures.

    par2
        Parameter containing list of second surface load values (if any).
        If Lab = CONV, the Par2 values are typically the bulk temperatures.
        Par2 is not used for other surface load labels.

    Notes
    -----
    Specifies a surface load "function" to be used when the SF or SFE
    command is issued.  The function is supplied through an array parameter
    vector which contains nodal surface load values.  Node numbers are
    implied from the sequential location in the array parameter.  For
    example, a value in location 11 applies to node 11.  The element faces
    are determined from the implied list of nodes when the SF or SFE
    command is issued.  Zero values should be supplied for nodes that have
    no load.  A tapered load value may be applied over the element face.
    These loads are in addition to any loads that are also specified with
    the SF or SFE commands.  Issue SFFUN (with blank remaining fields) to
    remove this specification.  Issue SFFUN,STATUS to list current
    settings.

    Starting array element numbers must be defined for each array parameter
    vector.  For example, SFFUN,CONV,A(1,1),A(1,2) reads the first and
    second columns of array A (starting with the first array element of
    each column) and associates the values with the nodes.  Operations
    continue on successive column array elements until the end of the
    column. Another example to show the order of the commands:

    SFFUN does not work for tabular boundary conditions.

    This command is also valid in PREP7.
    """
    command = f"SFFUN,{lab},{par},{par2}"
    return self.run(command, **kwargs)


def sfgrad(self, lab="", slkcn="", sldir="", slzer="", slope="", **kwargs):
    """Specifies a gradient (slope) for surface loads.

    APDL Command: SFGRAD

    Parameters
    ----------
    lab
        Valid surface load label.  Load labels are listed under "Surface
        Loads" in the input table for each element type in the Element
        Reference.

    slkcn
        Reference number of slope coordinate system (used with Sldir and
        SLZER to determine COORD).  Defaults to 0 (the global Cartesian
        coordinate system).

    sldir
        Slope direction in coordinate system SLKCN:

        X - Slope is along X direction (default).  Interpreted as R direction for non-
            Cartesian coordinate systems.

        Y - Slope is along Y direction.  Interpreted as  θ direction for non-Cartesian
            coordinate systems.

        Z - Slope is along Z direction.  Interpreted as Φ direction for spherical or
            toroidal coordinate systems.

    slzer
        Coordinate location (degrees for angular input) where slope
        contribution is zero (CVALUE = VALUE).  Allows the slope
        contribution to be shifted along the slope direction.  For angular
        input, SLZER should be between ±180° if the singularity [CSCIR]  is
        at 180° and should be between 0° and 360° if the singularity is at
        0°.

    slope
        Slope value (load per unit length or per degree).

    Notes
    -----
    Specifies a gradient (slope) for surface loads.  All surface loads
    issued with the SF, SFE, SFL, or SFA commands while this specification
    is active will have this gradient applied (for complex pressures, only
    the real component will be affected; for convections, only the bulk
    temperature will be affected).  The load value, CVALUE, calculated at
    each node is:

    CVALUE = VALUE + (SLOPE X (COORD-SLZER))

    where VALUE is the load value specified on the subsequent SF, SFE, SFL,
    or SFA commands and COORD is the coordinate value (in the Sldir
    direction of coordinate system SLKCN) of the node.  Only one SFGRAD
    specification may be active at a time (repeated use of this command
    replaces the previous specification with the new specification).  Issue
    SFGRAD (with blank fields) to remove the specification.  Issue
    SFGRAD,STAT to show the current command status.  The SFGRAD
    specification (if active) is removed when the LSREAD (if any) command
    is issued.

    SFGRAD does not work for tabular boundary conditions.

    This command is also valid in PREP7.
    """
    command = f"SFGRAD,{lab},{slkcn},{sldir},{slzer},{slope}"
    return self.run(command, **kwargs)


def sflist(self, node="", lab="", **kwargs):
    """Lists surface loads.

    APDL Command: SFLIST

    Parameters
    ----------
    node
        Node at which surface load is to be listed.  If ALL (or blank),
        list for all selected nodes [NSEL].  If NODE = P, graphical picking
        is enabled and all remaining command fields are ignored (valid only
        in the GUI).  A component name may be substituted for NODE.

    lab
        Valid surface load label.  If ALL (or blank), use all appropriate
        labels.

    Notes
    -----
    Lists the surface loads as applied with the SF command.  Loads are
    listed only for the specified nodes on external faces of selected area
    and volume elements.  Use SFELIST for line elements. The surface loads
    listed correspond to the current database values. The database is not
    updated for surface loads in POST1. Surface loads specified in tabular
    form, however, do list their values corresponding to the current
    results set in POST1.

    For SURF151 or SURF152 elements with an extra node for radiation and/or
    convection calculations (KEYOPT(5) = 1), the bulk temperature listed is
    the temperature of the extra node. If the thermal solution does not
    converge, the extra node temperature is not available for listing.

    This command is valid in any processor.
    """
    command = f"SFLIST,{node},{lab}"
    return self.run(command, **kwargs)


def sfscale(self, lab="", fact="", fact2="", **kwargs):
    """Scales surface loads on elements.

    APDL Command: SFSCALE

    Parameters
    ----------
    lab
        Valid surface load label.  If ALL, use all appropriate labels.

    fact
        Scale factor for the first surface load value.  Zero (or blank)
        defaults  to 1.0.  Use a small number for a zero scale factor.

    fact2
        Scale factor for the second surface load value.  Zero (or blank)
        defaults  to 1.0.  Use a small number for a zero scale factor.

    Notes
    -----
    Scales surface loads (pressure, convection, etc.) in the database on
    the selected elements.  Surface loads are applied with the SF, SFE, or
    SFBEAM commands.  Issue the SFELIST command to list the surface loads.
    Solid model boundary conditions are not scaled by this command, but
    boundary conditions on the FE model are scaled.

    Note:: : Such scaled FE boundary conditions may still be overwritten by
    unscaled solid model boundary conditions if a subsequent boundary
    condition transfer occurs.

    SFSCALE does not work for tabular boundary conditions.

    This command is also valid in PREP7 and in the /MAP processor.
    """
    command = f"SFSCALE,{lab},{fact},{fact2}"
    return self.run(command, **kwargs)



def kwpave(self, p1="", p2="", p3="", p4="", p5="", p6="", p7="", p8="",
           p9="", **kwargs):
    """Moves the working plane origin to the average location of keypoints.

    APDL Command: KWPAVE

    Parameters
    ----------
    p1, p2, p3, . . . , p9
        Keypoints used in calculation of the average.  At least one must be
        defined.  If P1 = P, graphical picking is enabled and all remaining
        command fields are ignored (valid only in the GUI).

    Notes
    -----
    Moves the origin of the working plane to the average of the specified
    keypoints.  Averaging is based on the active coordinate system.

    This command is valid in any processor.
    """
    command = f"KWPAVE,{p1},{p2},{p3},{p4},{p5},{p6},{p7},{p8},{p9}"
    return self.run(command, **kwargs)


def kwplan(self, wn="", korig="", kxax="", kplan="", **kwargs):
    """Defines the working plane using three keypoints.

    APDL Command: KWPLAN

    Parameters
    ----------
    wn
        Window number whose viewing direction will be modified to be normal
        to the working plane (defaults to 1).  If WN is a negative value,
        the viewing direction will not be modified.  If fewer than three
        points are used, the viewing direction of window WN will be used
        instead to define the normal to the working plane.

    korig
        Keypoint number defining the origin of the working plane coordinate
        system.  If KORIG = P, graphical picking is enabled and all
        remaining command fields are ignored (valid only in the GUI).

    kxax
        Keypoint number defining the x-axis orientation (defaults to the
        x-axis being parallel to the global X-axis; or if the normal to the
        working plane is parallel to the global X-axis, then defaults to
        being parallel to the global Y-axis).

    kplan
        Keypoint number defining the working plane (the normal defaults to
        the present display view [/VIEW] of window WN).

    Notes
    -----
    Defines a working plane to assist in picking operations using three
    keypoints as an alternate to the WPLANE command.  The three keypoints
    also define the working plane coordinate system.  A minimum of one
    keypoint (at the working plane origin) is required.  Immediate mode may
    also be active.  See WPSTYL command to set the style of working plane
    display.

    This command is valid in any processor.
    """
    command = f"KWPLAN,{wn},{korig},{kxax},{kplan}"
    return self.run(command, **kwargs)


def lwplan(self, wn="", nl1="", ratio="", **kwargs):
    """Defines the working plane normal to a location on a line.

    APDL Command: LWPLAN

    Parameters
    ----------
    wn
        Window number whose viewing direction will be modified to be normal
        to the working plane (defaults to 1).  If WN is a negative value,
        the viewing direction will not be modified.

    nl1
        Number of line to be used.  If NL1 = P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).

    ratio
        Location on NL1, specified as a ratio of the line length.  Must be
        between 0.0 and 1.0.  If RATIO = P, use graphical picking to
        specify location on the line.

    Notes
    -----
    Defines a working plane (to assist in picking operations) normal to a
    location on a line.  See WPSTYL command to set the style of working
    plane display.

    This command is valid in any processor.
    """
    command = f"LWPLAN,{wn},{nl1},{ratio}"
    return self.run(command, **kwargs)


def nwpave(self, n1="", n2="", n3="", n4="", n5="", n6="", n7="", n8="",
           n9="", **kwargs):
    """Moves the working plane origin to the average location of nodes.

    APDL Command: NWPAVE

    Parameters
    ----------
    n1, n2, n3, . . . , n9
        Nodes used in calculation of the average.  At least one must be
        defined.  If N1 = P, graphical picking is enabled and all remaining
        command fields are ignored (valid only in the GUI).

    Notes
    -----
    Averaging is based on the active coordinate system.

    This command is valid in any processor.
    """
    command = f"NWPAVE,{n1},{n2},{n3},{n4},{n5},{n6},{n7},{n8},{n9}"
    return self.run(command, **kwargs)


def nwplan(self, wn="", norig="", nxax="", nplan="", **kwargs):
    """Defines the working plane using three nodes.

    APDL Command: NWPLAN

    Parameters
    ----------
    wn
        Window number whose viewing direction will be modified to be normal
        to the working plane (defaults to 1).  If WN is a negative value,
        the viewing direction will not be modified.  If fewer than three
        points are used, the viewing direction of window WN will be used
        instead to define the normal to the working plane.

    norig
        Node number defining the origin of the working plane coordinate
        system.  If NORIG = P, graphical picking is enabled and all
        remaining command fields are ignored (valid only in the GUI).

    nxax
        Node number defining the x-axis orientation (defaults to the x-axis
        being parallel to the global X-axis; or if the normal to the
        working plane is parallel to the global X-axis, then defaults to
        being parallel to the global Y-axis).

    nplan
        Node number defining the working plane (the normal defaults to the
        present display view [/VIEW] of window WN).

    Notes
    -----
    Defines a working plane to assist in picking operations using three
    nodes as an alternate to the WPLANE command.  The three nodes also
    define the working plane coordinate system.  A minimum of one node (at
    the working plane origin) is required.  Immediate mode may also be
    active.  See the WPSTYL command to set the style of the working plane
    display.

    This command is valid in any processor.
    """
    command = f"NWPLAN,{wn},{norig},{nxax},{nplan}"
    return self.run(command, **kwargs)


def wpave(self, x1="", y1="", z1="", x2="", y2="", z2="", x3="", y3="",
          z3="", **kwargs):
    """Moves the working plane origin to the average of specified points.

    APDL Command: WPAVE

    Parameters
    ----------
    x1, y1, z1
        Coordinates (in the active coordinate system) of the first point.
        If X1 = P, graphical picking is enabled and all remaining command
        fields are ignored (valid only in the GUI).

    x2, y2, z2
        Coordinates (in the active coordinate system) of the second point.

    x3, y3, z3
        Coordinates (in the active coordinate system) of the third point.

    Notes
    -----
    Moves the origin of the working plane to the average of the specified
    points.  A point is considered specified only if at least one of its
    coordinates is non-blank, and at least one point (1, 2, or 3) must be
    specified.  Blank coordinates of a specified point are assumed to be
    zero.  Averaging is based on the active coordinate system.

    This command is valid in any processor.
    """
    command = f"WPAVE,{x1},{y1},{z1},{x2},{y2},{z2},{x3},{y3},{z3}"
    return self.run(command, **kwargs)


def wpcsys(self, wn="", kcn="", **kwargs):
    """Defines the working plane location based on a coordinate system.

    APDL Command: WPCSYS

    Parameters
    ----------
    wn
        Window number whose viewing direction will be modified to be normal
        to the working plane (defaults to 1).  If WN is a negative value,
        the viewing direction will not be modified.

    kcn
        Coordinate system number.  KCN may be 0,1,2 or any previously
        defined local coordinate system number (defaults to the active
        system).

    Notes
    -----
    Defines a working plane location and orientation based on an existing
    coordinate system.  If a Cartesian system is used as the basis (KCN)
    for the working plane, the working plane will also be Cartesian, in the
    X-Y plane of the base system.  If a cylindrical, spherical, or toroidal
    base system is used, the working plane will be a polar system in the
    R-θ plane of the base system.

    If working plane tracking has been activated (CSYS,WP or CSYS,4), the
    updated active coordinate system will be of a similar type, except that
    a toroidal system will be updated to a cylindrical system.  See the
    Modeling and Meshing Guide for more information on working plane
    tracking.

    This command is valid in any processor.

    Some primitive generation commands will not honor R-theta
    transformations for non-cartesian coordinate systems. Refer to the
    primitive commands table for more information.
    """
    command = f"WPCSYS,{wn},{kcn}"
    return self.run(command, **kwargs)


def wplane(self, wn="", xorig="", yorig="", zorig="", xxax="", yxax="",
           zxax="", xplan="", yplan="", zplan="", **kwargs):
    """Defines a working plane to assist in picking operations.

    APDL Command: WPLANE

    Parameters
    ----------
    wn
        Window number whose viewing direction will be modified to be normal
        to the working plane (defaults to 1).  If WN is a negative value,
        the viewing direction will not be modified.  If fewer than three
        points are used, the viewing direction of window WN will be used
        instead to define the normal to the working plane.

    xorig, yorig, zorig
        Global Cartesian coordinates of the origin of the working plane
        coordinate system.

    xxax, yxax, zxax
        Global Cartesian coordinates of a point defining the x-axis
        orientation.  The x-axis aligns with the projection of the line
        from this orientation point to the origin.

    xplan, yplan, zplan
        Global Cartesian coordinates of the third point defining the
        working plane.  This point will also define the location of the
        positive XY-sector of the working plane coordinate system.

    Notes
    -----
    Defines a working plane to assist in picking operations using the
    coordinates of three noncolinear points.  The three points also define
    the working plane coordinate system.  A minimum of one point (the
    working plane origin) is required.  Immediate mode may also be active.
    See WPSTYL command to set the style of working plane display.

    This command is valid in any processor.
    """
    command = f"WPLANE,{wn},{xorig},{yorig},{zorig},{xxax},{yxax},{zxax},{xplan},{yplan},{zplan}"
    return self.run(command, **kwargs)


def wpoffs(self, xoff="", yoff="", zoff="", **kwargs):
    """Offsets the working plane.

    APDL Command: WPOFFS

    Parameters
    ----------
    xoff, yoff, zoff
        Offset increments defined in the working plane coordinate system.
        If only ZOFF is used, the working plane will be redefined parallel
        to the present plane and offset by ZOFF.

    Notes
    -----
    Changes the origin of the working plane by translating the working
    plane along its coordinate system axes.

    This command is valid in any processor.
    """
    command = f"WPOFFS,{xoff},{yoff},{zoff}"
    return self.run(command, **kwargs)


def wprota(self, thxy="", thyz="", thzx="", **kwargs):
    """Rotates the working plane.

    APDL Command: WPROTA

    Parameters
    ----------
    thxy
        First rotation about the working plane Z axis (positive X toward
        Y).

    thyz
        Second rotation about working plane X axis (positive Y toward Z).

    thzx
        Third rotation about working plane Y axis (positive Z toward X).

    Notes
    -----
    The specified angles (in degrees) are relative to the orientation of
    the working plane.

    This command is valid in any processor.
    """
    command = f"WPROTA,{thxy},{thyz},{thzx}"
    return self.run(command, **kwargs)


def wpstyl(self, snap="", grspac="", grmin="", grmax="", wptol="",
           wpctyp="", grtype="", wpvis="", snapang="", **kwargs):
    """Controls the display and style of the working plane.

    APDL Command: WPSTYL

    Parameters
    ----------
    snap
        Snap increment for a locational pick (1E-6 minimum).  If -1, turn
        off snap capability.  For example, a picked location of 1.2456 with
        a snap of 0.1 gives 1.2, with 0.01 gives 1.25, with 0.001 gives
        1.246, and with 0.025 gives 1.250 (defaults to 0.05).

    grspac
        Graphical spacing between grid points.  For graphical
        representation only and not related to snap points  (defaults to
        0.1).

    grmin, grmax
        Defines the size of a square grid (if WPCTYP = 0) to be displayed
        over a portion of the working plane.  The opposite corners of the
        grid will be located at grid points nearest the working plane
        coordinates of (GRMIN,GRMIN) and (GRMAX,GRMAX).  If a polar system
        (WPCTYP = 1), GRMAX is the outside radius of grid and GRMIN is
        ignored.  If GRMIN = GRMAX, no grid will be displayed (defaults to
        -1.0 and 1.0 for GRMIN and GRMAX respectively).

    wptol
        The tolerance that an entity's location can deviate from the
        specified working plane, while still being considered on the plane.
        Used only for locational picking of vertices for polygons and
        prisms (defaults to 0.003).

    wpctyp
        Working plane coordinate system type:

        0 - Cartesian (default).  If working plane tracking is on [CSYS,4], the updated
            active coordinate system will also be Cartesian.

        1 - Polar.  If working plane tracking is on, the updated active coordinate system
            will be cylindrical.

        2 - Polar.  If working plane tracking is on, the updated active coordinate system
            will be spherical.

    grtype
        Grid type:

        0 - Grid and WP triad.

        1 - Grid only.

        2 - WP triad only (default).

    wpvis
        Grid visibility:

        0 - Do not show GRTYPE entities (grid and/or triad) (default).

        1 - Show GRTYPE entities.  Cartesian working planes will be displayed with a
            Cartesian grid, polar with a polar grid.

    snapang
        Snap angle (0--180) in degrees.  Used only if WPCTYP = 1 or 2.
        Defaults to 5 degrees.

    Notes
    -----
    Use WPSTYL,DEFA to reset the working plane to its default location and
    style.  Use WPSTYL,STAT to list the status of the working plane.  Blank
    fields will keep present settings.

    It is possible to specify SNAP and WPTOL values that will cause
    conflicts during picking operations. Check your values carefully, and
    if problems are noted, revert to the default values.

    WPSTYL with no arguments will toggle the grid on and off.  The working
    plane can be displayed in the non-GUI interactive mode only after
    issuing a /PLOPTS,WP,1 command.  See the Modeling and Meshing Guide for
    more information on working plane tracking.  See /PLOPTS command for
    control of hidden line working plane.

    This command is valid in any processor.
    """
    command = f"WPSTYL,{snap},{grspac},{grmin},{grmax},{wptol},{wpctyp},{grtype},{wpvis},{snapang}"
    return self.run(command, **kwargs)



def cvar(self, ir="", ia="", ib="", itype="", datum="", name="", **kwargs):
    """Computes covariance between two quantities.

    APDL Command: CVAR

    Parameters
    ----------
    ir
        Arbitrary reference number assigned to the resulting variable (2 to
        NV [NUMVAR]).  If this number is the same as for a previous
        variable, the previous variable will be overwritten with this
        result.

    ia, ib
        Reference numbers of the two variables to be operated on.  If only
        one, leave IB blank.

    itype
        Defines the type of response PSD to be calculated:

        0,1 - Displacement (default).

        2 - Velocity.

        3 - Acceleration.

    datum
        Defines the reference with respect to which covariance is to be
        calculated:

        1 - Absolute value.

        2 - Relative to base (default).

    name
        Thirty-two character name for identifying the variable on listings
        and displays.  Embedded blanks are compressed upon output.

    Notes
    -----
    This command computes the covariance value for the variables referenced
    by the reference numbers IA and IB.  If DATUM = 2, the variable
    referenced by IR will contain the individual modal contributions (i.e.,
    the dynamic or relative values).  If DATUM = 1, the variable referenced
    by IR will contain the modal contributions followed by the
    contributions of pseudo-static and covariance between dynamic and
    pseudo-static responses. File.PSD must be available for the
    calculations to occur.
    """
    command = f"CVAR,{ir},{ia},{ib},{itype},{datum},{name}"
    return self.run(command, **kwargs)


def pmgtran(self, fname="", freq="", fcnam1="", fcnam2="", pcnam1="",
            pcnam2="", ecnam1="", ccnam1="", **kwargs):
    """Summarizes electromagnetic results from a transient analysis.

    APDL Command: PMGTRAN

    Parameters
    ----------
    fname
        File name (8 characters maximum) to which tabular data and plot
        files will be written.  Must be enclosed in single quotes when the
        command is manually typed in.  Defaults to MG_TRNS.  The data file
        extension is .OUT and the plot file extension is .PLT.

    freq
        Frequency of solution output.  Defaults to 1.  Every FREQth
        solution on the results file is output.

    fcnam1, fcnam2
        Names of element components for force calculation.  Must be
        enclosed in single quotes when the command is manually typed in.

    pcnam1, pcnam2
        Names of element components for power loss calculation.  Must be
        enclosed in single quotes when the command is manually typed in.

    ecnam1, ccnam1
        Names of element components for energy and total current
        calculations, respectively.  Must be enclosed in single quotes when
        the command is manually typed in.

    Notes
    -----
    PMGTRAN invokes an ANSYS macro which calculates and summarizes
    electromagnetic results from a transient analysis.  The results are
    summarized by element components and listed on the screen as well as
    written to a file (Fname.OUT).  Also, graph plots of results as a
    function of time are created and written to a file (Fname.PLT) for use
    in the DISPLAY program.

    Two components may be selected for the summary of electromagnetic
    forces (see FMAGSUM), two for power loss, and one each for stored
    energy (see SENERGY) and total current (see CURR2D).  See the
    referenced commands for other restrictions.

    PMGTRAN is restricted to MKSA units.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"PMGTRAN,{fname},{freq},{fcnam1},{fcnam2},{pcnam1},{pcnam2},{ecnam1},{ccnam1}"
    return self.run(command, **kwargs)


def rcyc(self, ir="", ia="", sector="", name="", **kwargs):
    """Calculates cyclic results for a mode-superposition harmonic solution.

    APDL Command: RCYC

    Parameters
    ----------
    ir
        Arbitrary reference number assigned to the resulting variable (2 to
        NV [NUMVAR]). If this number is the same as for a previous
        variable, the previous variable will be overwritten with this
        result.

    ia
        Reference number of the variable to be operated on.

    sector
        Sector number to calculate the results for.

    name
        Thirty-two character name identifying the variable on listings and
        displays. Embedded blanks are compressed for output.

    Notes
    -----
    This command calculates the harmonic response in the sector specified
    by SECTOR for the variable referenced by the reference number IA. Only
    component values for IA are valid (no principles or sums). The variable
    specified by IR will contain the harmonic solution. Jobname.RFRQ from
    the cyclic mode-superposition harmonic solve and Jobname.RST or
    Jobname.RSTP from the cyclic modal solve must be available for the
    calculations to occur. The Jobname must be the same for the cyclic
    modal solve and the cyclic mode-superposition harmonic solve.

    For SECTOR > 1, the result is in the nodal coordinate system of the
    base sector, and it is rotated to the expanded sector’s location. Refer
    to Using the /CYCEXPAND Command in the Cyclic Symmetry Analysis Guide
    for more information.

    See also Mode-Superposition Harmonic Cyclic Symmetry Analysis in the
    Cyclic Symmetry Analysis Guide.
    """
    command = f"RCYC,{ir},{ia},{sector},{name}"
    return self.run(command, **kwargs)


def resp(self, ir="", lftab="", ldtab="", spectype="", dampratio="",
         dtime="", tmin="", tmax="", inputtype="", **kwargs):
    """Generates a response spectrum.

    APDL Command: RESP

    Parameters
    ----------
    ir
        Arbitrary reference number assigned to the response spectrum
        results (2 to NV [NUMVAR]).  If this number is the same as for a
        previously defined variable, the previously defined variable will
        be overwritten with these results.

    lftab
        Reference number of variable containing frequency table (created
        with FILLDATA or DATA command).  The frequency table defines the
        number and frequency of oscillating systems used to determine the
        response spectrum. The frequency interval need not be constant over
        the entire range. Frequencies must be input in ascending order.

    ldtab
        Reference number of variable containing the input time-history.

    spectype
        Defines the type of response spectrum to be calculated:

        0 or 1 - Displacement (relative to base excitation)

        2 - Velocity (relative to base excitation)

        3 - Acceleration response spectrum (absolute)

        4 - Pseudo-velocity

        5 - Pseudo-acceleration

    dampratio
        Ratio of viscous damping to critical damping (input as a decimal
        number).

    dtime
        Integration time step. This value should be equal to or greater
        than the integration time step used in the initial transient
        analysis performed to generate the input time-history (LDTAB).

    tmin, tmax
        Specifies a subset of the displacement-time history to be used in
        the response spectrum calculation.  Defaults to the full time
        range.

    inputtype
        Defines the type of the input time-history:

        0 - Displacement (default)

        1 - Acceleration

    Notes
    -----
    This command generates a response spectrum from a displacement or
    acceleration time-history and frequency data. The response spectrum is
    defined as the maximum response of single degree of freedom systems of
    varying frequency (or period) to a given input support excitation.

    A response spectrum analysis (ANTYPE, SPECTR with SPOPT, SPRS or MPRS)
    requires a response spectrum input. This input can be determined from
    the response spectrum printout or display of this command.

    If a response spectrum is to be calculated from a given displacement
    (or acceleration) time-history, the displacement time-history may be
    input to a single one-element reduced linear transient dynamic
    (ANTYPE,TRANS) analysis, so that the calculated output (which should be
    the same as the input) will be properly located on the file.

    The integration time step (argument DTIME on the RESP command) and the
    damping coefficient (argument dampRatio) are constant over the
    frequency range. The number of calculations done per response spectrum
    curve is the product of the number of input solution points (TMAX-
    TMIN)/DTIME and the number of frequency points (frequencies located in
    variable LFTAB).

     Input solution points requested (using DTIME and the frequency range)
    at a time not corresponding to an actual displacement solution time on
    the file are linearly interpolated with respect to the existing points.

    For the details of the response spectrum calculation, see POST26 -
    Response Spectrum Generator (RESP).
    """
    command = f"RESP,{ir},{lftab},{ldtab},{spectype},{dampratio},{dtime},{tmin},{tmax},{inputtype}"
    return self.run(command, **kwargs)


def rpsd(self, ir="", ia="", ib="", itype="", datum="", name="", signif="",
         **kwargs):
    """Calculates response power spectral density (PSD).

    APDL Command: RPSD

    Parameters
    ----------
    ir
        Arbitrary reference number assigned to the resulting variable (2 to
        NV [NUMVAR]).  If this number is the same as for a previous
        variable, the previous variable will be overwritten with this
        result.

    ia, ib
        Reference numbers of the two variables to be operated on.  If only
        one, leave IB blank.

    itype
        Defines the type of response PSD to be calculated:

        0,1 - Displacement (default).

        2 - Velocity.

        3 - Acceleration.

    datum
        Defines the reference with respect to which response PSD is to be
        calculated:

        1 - Absolute value.

        2 - Relative to base (default).

    name
        Thirty-two character name identifying variable on listings and
        displays.  Embedded blanks are compressed for output.

    signif
        Combine only those modes whose significance level exceeds the
        SIGNIF threshold. The significance level is defined as the modal
        covariance matrix term divided by the maximum of all the modal
        covariance matrix terms. Any term whose significance level is less
        than SIGNIF is considered insignificant and does not contribute to
        the response. All modes are taken into account by default (SIGNIF =
        0.0).

    Notes
    -----
    This command calculates response power spectral density (PSD) for the
    variables referenced by the reference numbers IA and IB.  The variable
    referred by IR will contain the response PSD.  You must issue the
    STORE,PSD command first; File.PSD must be available for the
    calculations to occur.

    See POST26 - Response Power Spectral Density in the Mechanical APDL
    Theory Reference for more information on these equations.
    """
    command = f"RPSD,{ir},{ia},{ib},{itype},{datum},{name},{signif}"
    return self.run(command, **kwargs)


def smooth(self, vect1="", vect2="", datap="", fitpt="", vect3="",
           vect4="", disp="", **kwargs):
    """Allows smoothing of noisy data and provides a graphical representation

    APDL Command: SMOOTH
    of the data.

    Parameters
    ----------
    vect1
        Name of the first vector that contains the noisy data set (i.e.,
        independent variable). You must create and fill this vector before
        issuing SMOOTH.

    vect2
        Name of the second vector that contains the dependent set of data.
        Must be the same length as the first vector. You must create and
        fill this vector before issuing SMOOTH.

    datap
        Number of data points to be fitted, starting from the beginning of
        the vector. If left blank, the entire vector will be fitted. The
        maximum number of data points is 100,000 (or greater, depending on
        the memory of the computer).

    fitpt
        Order of the fitting curve that will be used as a smooth
        representation of the data. This number should be less than or
        equal to the number of the data points. Default (blank) is one-half
        the number of data points. Maximum number of smoothed data fitting
        order is the number of data points up to 50.  Depending on this
        number, the smoothed curve will be one of the following:

        1 - Curve is the absolute average of all of the data points.

        2 - Curve is the least square average of all of the data points.

        3 or more - Curve is a polynomial of the order (n-1), where n is the number of data fitting
                    order points.

    vect3
        Name of the vector that contains the smoothed data of the
        independent variable. This vector should have a length equal to or
        greater than the number of smoothed data points. In batch (command)
        mode, you must create this vector before issuing the SMOOTH
        command. In interactive mode, the GUI automatically creates this
        vector (if it does not exist). If you do not specify a vector name,
        the GUI will name the vector smth_ind.

    vect4
        Name of the vector that contains the smoothed data of the dependent
        variable.  This vector must be the same length as Vect3.  In batch
        (command) mode, you must create this vector before issuing the
        SMOOTH command. In interactive mode, the GUI automatically creates
        this vector (if it does not exist). If you do not specify a vector
        name, the GUI will name the vector smth_dep.

    disp
        Specifies how you want to display data. No default; you must
        specify an option.

        1 - Unsmoothed data only

        2 - Smoothed data only

        3 - Both smoothed and unsmoothed data

    Notes
    -----
    You can control the attributes of the graph using standard ANSYS
    controls (/GRID, /GTHK, /COLOR, etc.). If working interactively, these
    controls appear in this dialog box for convenience, as well as in their
    standard dialog boxes. You must always create Vect1 and Vect2 (using
    *DIM) and fill these vectors before smoothing the data. If you're
    working interactively, ANSYS automatically creates Vect3 and Vect4, but
    if you're working in batch (command) mode, you must create Vect3 and
    Vect4 (using *DIM) before issuing SMOOTH.  Vect3 and Vect4 are then
    filled automatically by ANSYS.  In addition, ANSYS creates an
    additional TABLE type array that contains the smoothed array and the
    unsmoothed data to allow for plotting later with *VPLOT.  Column 1 in
    this table corresponds to Vect1, column 2 to Vect2, and column 3 to
    Vect4.  This array is named Vect3_SMOOTH, up to a limit of 32
    characters. For example, if the array name is X1, the table name is
    X1_SMOOTH.

    This command is also valid in PREP7 and SOLUTION.
    """
    command = f"SMOOTH,{vect1},{vect2},{datap},{fitpt},{vect3},{vect4},{disp}"
    return self.run(command, **kwargs)


def vget(self, par="", ir="", tstrt="", kcplx="", **kwargs):
    """Moves a variable into an array parameter vector.

    APDL Command: VGET

    Parameters
    ----------
    par
        Array parameter vector in the operation.

    ir
        Reference number of the variable (1 to NV [NUMVAR]).

    tstrt
        Time (or frequency) corresponding to start of IR data.  If between
        values, the nearer value is used.

    kcplx
        Complex number key:

        * ``0`` - Use the real part of the IR data.

        * ``1`` - Use the imaginary part of the IR data.

    Notes
    -----
    Moves a variable into an array parameter vector.  The starting array
    element number must be defined.  For example, VGET,A(1),2 moves
    variable 2 (starting at time 0.0) to array parameter A.  Looping
    continues from array element A(1) with the index number incremented by
    one until the variable is filled.  The number of loops may be
    controlled with the *VLEN command (except that loop skipping (NINC) is
    not allowed).  For multi-dimensioned array parameters, only the first
    (row) subscript is incremented.
    """
    command = f"VGET,{par},{ir},{tstrt},{kcplx}"
    return self.run(command, **kwargs)


def vput(self, par="", ir="", tstrt="", kcplx="", name="", **kwargs):
    """Moves an array parameter vector into a variable.

    APDL Command: VPUT

    Parameters
    ----------
    par
        Array parameter vector in the operation.

    ir
        Arbitrary reference number assigned to this variable (1 to NV
        [NUMVAR]).  Overwrites any existing results for this variable.

    tstrt
        Time (or frequency) corresponding to start of IR data.  If between
        values, the nearer value is used.

    kcplx
        Complex number key:

        0 - Use the real part of the IR data.

        1 - Use the imaginary part of the IR data.

    name
        Thirty-two character name identifying the item on printouts and
        displays. Defaults to the label formed by concatenating VPUT with
        the reference number IR.

    Notes
    -----
    At least one variable should be defined (NSOL, ESOL, RFORCE, etc.)
    before using this command.  The starting array element number must be
    defined.  For example, VPUT,A(1),2 moves array parameter A to variable
    2 starting at time 0.0.  Looping continues from array element A(1) with
    the index number incremented by one until the variable is filled.
    Unfilled variable locations are assigned a zero value.  The number of
    loops may be controlled with the *VLEN command (except that loop
    skipping (NINC) is not allowed).  For multi-dimensioned array
    parameters, only the first (row) subscript is incremented.
    """
    command = f"VPUT,{par},{ir},{tstrt},{kcplx},{name}"
    return self.run(command, **kwargs)



def cformat(self, nfirst="", nlast="", **kwargs):
    """Controls the graphical display of alphanumeric character strings for

    APDL Command: /CFORMAT
    parameters, components, assemblies, and tables.

    Parameters
    ----------
    nfirst
        Display the first n characters of the parameter, component,
        assembly, or table name, up to 32. Defaults to 32.

    nlast
        Display the last n characters of the parameter, component,
        assembly, or table name, up to 32. Defaults to 0.

    Notes
    -----
    Use this command to control the length of the character string that is
    shown in the graphics window for a parameter, component, assembly, or
    table name.

    The total number of characters (NFIRST + NLAST +3) cannot exceed 32.

    If NFIRST is greater than zero and NLAST = 0, only the NFIRST
    characters are displayed, followed by an ellipsis.

    If NFIRST = 0 and NLAST is greater than zero, only the NLAST characters
    are displayed, preceded by an ellipsis (...).

    If both NFIRST and NLAST are greater than zero, the name will be shown
    as NFIRST, followed by an ellipsis (...), followed by NLAST, up to a
    maximum of 32 characters.

    For example, if NFIRST = 6 and NLAST = 3, and the character string is
    LENGTHOFSIDEONE, then it will appear in the graphics window as
    LENGTH...ONE.

    If the actual length of the character string is less than the specified
    combination of NFIRST + NLAST +3, then the actual string will be used.

    This command is valid in any processor.
    """
    command = f"/CFORMAT,{nfirst},{nlast}"
    return self.run(command, **kwargs)


def clabel(self, wn="", key="", **kwargs):
    """Specifies contour labeling.

    APDL Command: /CLABEL

    Parameters
    ----------
    wn
        Window number (or ALL) to which command applies (defaults to 1).

    key
        Labeling key:

        0 or 1 - Label contours with legend or color (default).

        -1 - No contour labeling.

        N - Same as 1 except show alphabetic legend only on every Nth element.

    Notes
    -----
    Labels contours for identification with alphabetic legend for vector
    displays and color for raster displays.  Number of contours is
    automatically reduced to 9 (or fewer) for clarity.  Use /CONTOUR
    command to increase (24 maximum for alphabetic labeling; no limit for
    color labeling).

    This command is valid in any processor.
    """
    command = f"/CLABEL,{wn},{key}"
    return self.run(command, **kwargs)


def contour(self, wn="", ncont="", vmin="", vinc="", vmax="", **kwargs):
    """Specifies the uniform contour values on stress displays.

    APDL Command: /CONTOUR

    Parameters
    ----------
    wn
        Window number (or ALL) to which command applies (defaults to 1).

    ncont
        Number of contour values. NCONT defaults to 9 for X11 or WIN32 and
        to 128 for X11c or WIN32C.  The default graphics window display for
        3-D devices is a smooth continuous shading effect that spans the
        maximum of 128 contours available. Use the /DV3D command to create
        defined banding for your contour values (values of 9 and 128 are
        displayed in smooth shading only). The legend, however, will
        display only nine color boxes, which span the full range of colors
        displayed in the graphics window.

    vmin
        Minimum contour value.  If VMIN = AUTO, automatically calculate
        contour values based upon NCONT uniformly spaced values over the
        min-max extreme range.  Or, if VMIN = USER, set contour values to
        those of the last display (useful when last display automatically
        calculated contours).

    vinc
        Value increment (positive) between contour values.  Defaults to
        (VMAX-VMIN)/NCONT.

    vmax
        Maximum contour value.  Ignored if both VMIN and VINC are
        specified.

    Notes
    -----
    See the /CVAL command for alternate specifications.  Values represent
    contour lines in vector mode, and the algebraic maximum of contour
    bands in raster mode.

    Note:: : No matter how many contours (NCONT) are specified by /CONTOUR,
    the actual number of contours that appear on your display depends also
    on the device name, whether the display is directed to the screen or to
    a file, the display mode (vector or raster), and the number of color
    planes.  (All these items are controlled by /SHOW settings.) In any
    case, regardless of whether they are smoothed or banded, only 128
    contours can be displayed. See Creating Geometric Results Displays in
    the Basic Analysis Guide for more information on changing the number of
    contours.

    If the current ANSYS graphics are not displayed as Multi-Plots, then
    the following is true:  If the current device is a 3-D device
    [/SHOW,3D], the model contours in all active windows will be the same,
    even if separate /CONTOUR commands are issued for each active window.
    For efficiency, ANSYS 3-D graphics logic maintains a single data
    structure (segment), which contains precisely one set of contours.  The
    program displays the same segment in all windows.  The view settings of
    each window constitute the only differences in the contour plots in the
    active windows.

    This command is valid in any processor.
    """
    command = f"/CONTOUR,{wn},{ncont},{vmin},{vinc},{vmax}"
    return self.run(command, **kwargs)


def cval(self, wn="", v1="", v2="", v3="", v4="", v5="", v6="", v7="",
         v8="", **kwargs):
    """Specifies nonuniform contour values on stress displays.

    APDL Command: /CVAL

    Parameters
    ----------
    wn
        Window number (or ALL) to which command applies (defaults to 1).

    v1, v2, v3, . . . , v8
        Up to 8 contour values may be specified (in ascending order).  The
        0.0 value (if any) must not be the last value specified.  If no
        values are specified, all contour specifications are erased and
        contours are automatically calculated.

    Notes
    -----
    This command is similar to the /CONTOUR command. With /CVAL, however,
    you define the upper level of each contour band instead of having the
    contours uniformly distributed over the range. The minimum value
    (including a zero value for the first band) for a contour band cannot
    be specified. If you use both /CONTOUR and /CVAL, the last command
    issued takes precedence.

    This command is valid in any processor.
    """
    command = f"/CVAL,{wn},{v1},{v2},{v3},{v4},{v5},{v6},{v7},{v8}"
    return self.run(command, **kwargs)


def gformat(self, ftype="", nwidth="", dsignf="", **kwargs):
    """Specifies the format for the graphical display of numbers.

    APDL Command: /GFORMAT

    Parameters
    ----------
    ftype
        FORTRAN format types (G is the default if this field is left
        blank.)

        G - Gxx.yy.  xx and yy are described below.

        F - Fxx.yy

        E - Exx.yy

    nwidth
        Total width (12 maximum) of the field (the xx in Ftype).  Defaults
        to 12.

    dsignf
        Number of digits after the decimal point (yy in F or E format) or
        number of significant digits in G format.  Range is 1 to xx-6 for
        Ftype = G or E; and 0 to xx-3 for Ftype = F.  The default is a
        function of Ftype and NWIDTH.

    Notes
    -----
    Lets you control the format of the graphical display of floating point
    numbers.   Issue /GFORMAT,STAT to display the current settings; issue
    /GFORMAT,DEFA to let ANSYS choose the format for the graphical display
    of floating numbers.

    This command is valid in any processor.
    """
    command = f"/GFORMAT,{ftype},{nwidth},{dsignf}"
    return self.run(command, **kwargs)


def hbc(self, wn="", key="", **kwargs):
    """Determines how boundary condition symbols are displayed in a display

    APDL Command: /HBC
    window.

    Parameters
    ----------
    wn
        Window reference number. This number can be any window numbered 1
        to 5, or ALL (for all active windows). Defaults to 1

    key
        Key to enable/disable hidden surface boundary condition symbol
        display for 2-D graphics devices and to request improved pressure
        contour display for 2-D and 3-D devices: Key = ON, YES or 1 will
        show your BC symbols on the hidden surfaces and use an improved
        pressure contour display. Key = OFF, NO or 0 (default) will hide
        the symbols .
    """
    command = f"/HBC,{wn},{key}"
    return self.run(command, **kwargs)


def number(self, nkey="", **kwargs):
    """Specifies whether numbers, colors, or both are used for displays.

    APDL Command: /NUMBER

    Parameters
    ----------
    nkey
        Numbering style:

         0  - Color (terminal dependent) the numbered items and show numbers.

         1  - Color the numbered items.  Do not show the numbers.

         2  - Show the numbers.  Do not color the items.

        -1  - Do not color the items or show the numbers. For contour plots, the resulting
              display will vary (see below).

    Notes
    -----
    Specifies whether numbers, colors, or both are used for numbering
    displays [/PNUM] of nodes, elements, keypoints, lines, areas, and
    volumes.

      Shading is also available for terminals configured with more than 4
    color planes [/SHOW].  Color automatically appears for certain items
    and may be manually controlled (off or on) for other items.

    When you suppress color (NKEY = -1) your contour plots will produce
    different results, depending on your graphics equipment. For non-3-D
    devices (X11, Win32, etc.) your contour plot will be white (no color).
    For 3-D devices, such as OpenGL, the resulting display will be in
    color.

    The following items are automatically given discrete colors:  Boundary
    condition symbols [/PBC], curves on graph displays, and distorted
    geometry on postprocessing displays.  Contour lines in postprocessing
    displays are automatically colored based upon a continuous, rather than
    a discrete, spectrum so that red is associated with the highest contour
    value.  On terminals with raster capability [/SHOW], the area between
    contour lines is filled with the color of the higher contour.

    Explicit entity colors or the discrete color mapping may be changed
    with the /COLOR command.

    This command is valid in any processor.
    """
    command = f"/NUMBER,{nkey}"
    return self.run(command, **kwargs)


def pbc(self, item="", key="", min_="", max_="", abs_="", **kwargs):
    """Shows boundary condition (BC) symbols and values on displays.

    APDL Command: /PBC

    Parameters
    ----------
    item
        Label identifying the item:

        U - Applied translational constraints (UX, UY, UZ).

        ROT - Applied rotational constraints (ROTX, ROTY, ROTZ).

        TEMP - Applied temperatures (TEMP, TBOT, TE2, TE3, . . ., TTOP).

        PRES - Applied fluid pressures.

        V - Applied flow velocities (VX, VY, VZ).

        VOLT - Applied voltages.

        MAG - Applied scalar magnetic potentials.

        A - Applied vector magnetic potentials.

        CONC - Concentration.

        CHRG - Applied electric charge.

        F or FORC - Applied structural forces (FX, FY, FZ).

        M or MOME - Applied structural moments (MX, MY, MZ).

        HEAT - Applied heat flows (HEAT, HBOT, HE2, HE3, . . ., HTOP).

        FLOW - Applied fluid flow.

        AMPS - Applied current flow.

        FLUX - Applied magnetic flux.

        CSG - Applied magnetic current segments.

        RATE - Diffusion flow rate.

        MAST - Master degrees of freedom.

        CP - Coupled nodes.

        CE - Nodes in constraint equations.

        NFOR - POST1 nodal forces.

        NMOM - POST1 nodal moments

        RFOR - POST1 reaction forces.

        RMOM - POST1 reaction moments (MX, MY, MZ).

        PATH - Path geometry (undistorted) associated with the PATH command after a PDEF or
               PVECT command has been issued.

        ACEL - Global acceleration (ACELX, ACELY, ACELZ vector).

        OMEG - Global angular velocity (OMEGX, OMEGY, OMEGZ vector) and acceleration (DOMEGX,
               DOMEGY, DOMEGZ vector).

        WELD - Applied spotwelds (ANSYS LS-DYNA).

        ALL - Represents all appropriate labels.

    key
        Symbol key:

        0 - Do not show symbol.

        1 - Show symbol.

        2 - Plot value next to symbol.

    min\_
        Minimum value in a range of values plotted on screen.

    max\_
        Maximum value in a range of values plotted on screen.

    abs\_
        Absolute number.  If KEY = 2 and ABS = 0, a number falling between
        the MIN and MAX is displayed.  If ABS is not specified, it defaults
        to 0.  If KEY = 2 and ABS = 1, an absolute value falling between
        the MIN and MAX is displayed.   ABS = 1 lets you eliminate the
        display of numbers whose absolute values are less than a desired
        tolerance.  For example, if ABS = 1, MIN = 10 and MAX = 1e8, values
        such as .83646 and -5.59737 are not displayed.

    Notes
    -----
    The /PBC command adds degree of freedom constraint, force load, and
    other symbols to displays.

    Symbols are applied to the selected nodes only.  All arrow and
    arrowhead symbols are oriented in the nodal coordinate system and lie
    in two perpendicular planes.  Force arrows are scaled proportional to
    their magnitude. (If KEY =  1, use /VSCALE to change arrow length.)
    For scalar quantities, the specific component direction (i.e., x, y, or
    z) of the symbol has no meaning, but the positive or negative sense
    (e.g., positive or negative x) represents a positive or negative scalar
    value, respectively.

    The effects of the /PBC command are not cumulative (that is, the
    command does not modify an existing setting from a previously issued
    /PBC command).  If you issue multiple /PBC commands during an analysis,
    only the setting specified by the most recent /PBC command applies.

    Use /PSTATUS or /PBC,STAT to display settings.  Use /PBC,DEFA to reset
    all specifications back to default.  See the /PSF and /PBF commands for
    other display symbols.

    In a cyclic symmetry analysis, the /PBC command is deactivated when
    cyclic expansion is active (/CYCEXPAND,,ON). To view boundary
    conditions on the basic sector, deactivate cyclic expansion
    (/CYCEXPAND,,OFF) and issue this command:: : /PBC,ALL,,1

    Issuing the command /PBC,PATH, ,1 displays all defined paths.

    The /PBC command is valid in any processor.
    """
    command = f"/PBC,{item},,{key},{min_},{max_},{abs_}"
    return self.run(command, **kwargs)


def pbf(self, item="", key="", **kwargs):
    """Shows magnitude of body force loads on displays.

    APDL Command: /PBF

    Parameters
    ----------
    item
        Label identifying the item:

        TEMP - Applied temperatures.

        FLUE - Applied fluences.

        HGEN - Applied heat generation rates.

        JS - Applied current density magnitude.

        JSX - X-component of current density.

        JSY - Y-component of current density.

        JSZ - Z-component of current density.

        PHASE - Phase angle of applied load.

        MVDI - Applied magnetic virtual displacements flag.

        CHRGD - Applied electric charge density.

        VLTG - Applied voltage drop.

    key
        Symbol key:

        0 - Do not show body force load contours.

        1 - Show body force load contours.

        2 - Show current density as a vector (not a contour).

    Notes
    -----
    Shows body force loads as contours on displays for the selected
    elements.

    The effects of the /PBF command are not cumulative (that is, the
    command does not modify an existing setting from a previously issued
    /PBF command).  If you issue multiple /PBF commands during an analysis,
    only the setting specified by the most recent /PBF command applies.

    Use /PSTATUS or /PBF,STAT to display settings.  Use /PBF,DEFA to reset
    all specifications back to default.  See also the /PSF and /PBC command
    for other display contours.

    Portions of this command are not supported by PowerGraphics
    [/GRAPHICS,POWER].

    This command is valid in any processor.
    """
    return self.run(f"/PBF,{item},,{key}", **kwargs)


def plopts(self, label="", key="", **kwargs):
    """Controls graphics options on subsequent displays.

    APDL Command: /PLOPTS

    Parameters
    ----------
    label
        Apply display items as selected from the following labels:

        INFO - Controls the display of the legend (ON or OFF) and allows the choice of preset
               or Multi-legend placement. Control is provided by the KEY
               values. (Defaults to KEY=3 when the GUI is on. Defaults to
               KEY= 2 otherwise.)

        LEG1 - Header portion of legend column (defaults to ON).

        LEG2 - View portion of legend column (defaults to ON (except off with contour
               displays)).

        LEG3 - View the contour section of the legend column (defaults to ON).

        FRAME - Frame border lines around windows (defaults to ON).

        TITLE - Title (bottom left text) (defaults to ON).

        MINM - Min-Max symbols on contour displays (defaults to ON).

        LOGO - ANSYS logo (defaults to OFF (displayed as text at top of legend column)).  If
               KEY = ON, the text is removed from legend column but the
               logo symbol is displayed in whichever active window is
               either in the uppermost right corner and on top, or if there
               is no window in that location, then in the window to the
               furthest right of the screen.  Version information remains
               in the legend column.

        WINS - Controls whether graphics windows automatically stretch or shrink to adjust to
               screen size as the legend column is turned off or on
               [/PLOPTS,INFO] (defaults to ON).  If WINS is on and the
               legend column is changed from off to on, all windows are
               shrunk regardless of what their correct size is.

        WP - Working plane (defaults to OFF).  The working plane is drawn as part of the
             display (not just an overlaid image as in WPSTYL).  This
             option is best used in combination with a hidden-line
             technique [/TYPE].

        DATE - Controls the display of the date and time in your legend. Subsequent KEY values
               control the display as follows:

        FILE - Controls the display of the ANSYS jobname in your legend. Subsequent KEY values
               control the display as follows:

    key
        Switch:

        OFF or 0 - Do not apply this display item. For Label = DATE, no time or date are
                   displayed.

        ON or 1 - Apply this display item. For Label = DATE, show only the date.

        AUTO or 2 - For Label = INFO, initiate Auto-legend mode.  If the display has contours, the
                    legend is ON; if the display has no contours, the
                    legend is OFF. For Label = DATE, display both the date
                    and time.

        3 - For Label = INFO , switch to Multi-legend mode. See the /UDOC command for the
            available legend configurations.

    Notes
    -----
    Use /PLOPTS,STAT to display settings.  Use /PLOPTS,DEFA to reset all
    specifications back to their defaults.

    When you perform multiple results displays, contours on the legend
    column may be truncated.  To avoid this, specify /PLOPTS,LEG1,0.

    The Multi-legend mode provides a number of legend data item priority
    and placement options. These options are accessed via the GUI at
    Utility Menu> PlotCtrls> Window Controls> Window Options. The /UDOC
    command provides command line options for this capability.

    This command is valid in any processor.

    This command is not available for Academic Research or Teaching level
    products
    """
    command = f"/PLOPTS,{label},{key}"
    return self.run(command, **kwargs)


def pnum(self, label="", key="", **kwargs):
    """Controls entity numbering/coloring on plots.

    APDL Command: /PNUM

    Parameters
    ----------
    label
        Type of numbering/coloring:

        NODE - Node numbers on node and element plots.

        ELEM - Element numbers and colors on element plots.

        SEC - Section numbers and colors on element and solid model plots (see "Notes").

        MAT - Material set numbers and colors on element and solid model plots  (see
              "Notes").

        TYPE - Element type reference numbers and colors on element and solid model plots (see
               "Notes").

        REAL - Real constant set numbers and colors on element and solid model plots (see
               "Notes").

        ESYS - Element coordinate system numbers on element and solid model plots (see
               "Notes").

        PART - Element part numbers and colors on element plots (applicable to ANSYS LS-DYNA
               only).

        LOC - Location numbers/colors of the element in matrix assembly order on element
              plots.

        Note:LOC and ELEM numbers will be the same unless the model has been reordered. - KP

        Keypoint numbers on solid model plots. - LINE

        Line numbers on solid model plots (both numbers and colors on line plots). - AREA

        Area numbers on solid model plots (both numbers and colors on area plots). - VOLU

        Volume numbers on solid model plots (both numbers and colors on volume plots). - SVAL

        Stress (or any contour) values on postprocessing plots, and surface load values and colors on model plots (when surface load symbols are on [/PSF]).  For tabular boundary conditions, the table-evaluated values will be displayed on node, element, or contour displays in POST1 when load symbols (/PBF, /PSF, /PBC) are on and TABNAM is OFF. - TABNAM

        Table names for tabular boundary conditions.  If this label is turned on, the table name appears next to the appropriate symbol, arrow, face outline, or contour as dictated by the /PSF, /PBC, and /PBF commands. - STAT

        Shows current settings for /PNUM. - DEFA

    key
        Switch:

        0 - Turns OFF numbers/colors for specified label.

        1 - Turns ON numbers/colors for specified label.

    Notes
    -----
    This command specifies entity numbering and coloring for subsequent
    plots.

    The MAT, TYPE, REAL, and ESYS labels activate both the numbering and
    coloring of the corresponding attributes for  EPLOT, KPLOT, LPLOT,
    APLOT, and VPLOT. The ELEM, MAT, TYPE, REAL, ESYS, PART (ANSYS LS-DYNA
    only), and LOC labels are mutually exclusive, i.e., only one can be
    specified at a time. Also, turning on a LINE, AREA, or VOLU label will
    turn off the MAT, TYPE, REAL, and PART labels.

    PowerGraphics [/GRAPHICS,POWER] displays for/PNUM can be problematic.
    /PNUM,ELEM will display erratically depending on other display command
    specifications, while /PNUM,LOC and /PNUM,ESYS are not supported.

    Element and volume numbers are not visible for 3-D elements and volumes
    when Z-buffering is turned on (/TYPE,,[6,7, or 8]).

    Use /PSTATUS or /PNUM,STAT to show settings. Use /PNUM,DEFA to reset
    all specifications back to default. Use the /NUMBER command to control
    whether numbers and colors are displayed together.

    This command is valid in any processor
    """
    command = f"/PNUM,{label},{key}"
    return self.run(command, **kwargs)


def psf(self, item="", comp="", key="", kshell="", color="", **kwargs):
    """Shows surface load symbols on model displays.

    APDL Command: /PSF

    Parameters
    ----------
    item, comp
        Labels identifying the surface load to be shown; see
        Table 227: /PSF - Valid Item and Component Labels.

    key
        Key to turn surface load symbols on or off:

        0 - Off (default).

        1 - On, shown as face outlines.  Line surface loads (SFL) on solid model plots are
            shown as arrows.

        2 - On, shown as arrows.

        3 - On, shown as color filled surfaces.  Line and area surface loads (SFL and SFA)
            on solid model plots are shown as arrows.

    kshell
        Visibility key for shell elements.

        0 - Off (default),  surface load symbols are displayed only on visible load faces.

        1 - On, surface load symbols are displayed even if load face is not visible.

    color
        Visibility key for contour legend.

        ON - The symbols (arrows or face outlines) will show up in color with the legend
             showing the corresponding color labels (default).

        OFF - The contour legend will not be displayed. The symbols (arrows or face outlines)
              will show up in grey. The size of the arrows will be
              proportional to the applied load.

    Notes
    -----
    The /PSF command determines whether and how to show surface loads on
    subsequent model displays.

    If surface loads are applied to solid model entities, only solid model
    plots show the load symbols; node and element plots do not show them
    unless the loads are transferred (SFTRAN or SBCTRAN).  Similarly, solid
    model plots do not show the load symbols if surface loads are applied
    to nodes and elements.  For node and element plots of shell element
    models, the surface load symbols are shown only if the load face is
    visible from the current viewing direction.

    The effects of the /PSF command are not cumulative (that is, the
    command does not modify an existing setting from a previously issued
    /PSF command).  Only the setting specified via the most recent /PSF
    command applies.

    If you issue a postprocessing (POST1) plot command that produces result
    contours (such as PLNSOL), the /PSF command has no effect. This
    behavior prevents conflicting contours in the graphics window.

    When using the radiosity method (Item = RDSF and Comp = ENCL) with Key
    = 2, the radiation arrows point outward from any element face. When
    using SURF154 with KEYOPT(2) = 1, set the Item to PRES and leave the
    Component Label blank.

    /PSF,STAT displays current /PSF settings, and /PSF,DEFA resets them
    back to default.

    Other useful commands are /PNUM,SVAL,1 to show the values of the
    surface loads, /VSCALE to change arrow lengths, and /PBC and /PBF to
    activate other load symbols.

    For beam elements, only the colors representing shear (GREEN) and
    normal (RED) pressures are displayed for the arrows. The color of these
    arrows does not correspond to the magnitudes in the contour legend. The
    length of these arrows does, however, correlate to the relative
    magnitude of the pressures.

    For elements SURF159, SOLID272, SOLID273, PIPE288 and PIPE289, the /PSF
    command is not available when displaying elements with shapes
    determined from the real constants or section definition (/ESHAPE). For
    PIPE288 and PIPE289, only external loads applied via the SFBEAM command
    are displayed.

    This command is valid in any processor.

    Table: 227:: : /PSF - Valid Item and Component Labels

    Pressure loads apply to the element coordinate system (KEYOPT(2) = 0).
    Adjust appropriately for a local coordinate system (KEYOPT(2) = 1). See
    Figure: 153.2:: Pressures in the Element Reference.
    """
    command = f"/PSF,{item},{comp},{key},{kshell},{color}"
    return self.run(command, **kwargs)


def psymb(self, label="", key="", **kwargs):
    """Shows various symbols on displays.

    APDL Command: /PSYMB

    Parameters
    ----------
    label
        Show symbols as selected from the following labels:

        CS - Local coordinate systems.

        NDIR - Nodal coordinate systems (on rotated nodes only).

        ESYS - Element coordinate systems (element displays only).

        LDIR - Line directions (line displays only).

        LDIV - Controls the display of element divisions on lines.

        ADIR - Area direction symbol (for keypoint, line, area and volume plots).

        LAYR - Layer orientations (relative to the projected element x-axis) or fiber
               orientations in smeared reinforcing elements. Used only
               within an element display. Use KEY to specify the layer
               number.

        ECON - Element mesh symbols on keypoints and lines.

        DOT - Larger symbols displayed for node and keypoint locations.  When Label = DOT,
              KEY = 1 by default.

        XNOD - Extra node of surface or circuit elements.

        FBCS - Force boundary condition scaling. Subsequent KEY value determines whether or
               not to scale the applied and derived forces/moments to the
               same maximum value.

        DEFA - Resets the symbol keys so that ANSYS displays none of the symbols controlled by
               the /PSYMB command.  The value of the KEY field is ignored.

        STAT - Prints the status of the settings of the symbol keys controlled by the /PSYMB
               command.  The KEY field is ignored.

    key
        Symbol key:

        -1  - Effective only if Label = LAYR and solid shape element display (/ESHAPE) is
              active. Orientation of all layers appears with the solid
              shape element display.

        0  - No symbol (default). If Label = LDIV, then KEY= 0 indicates that the displayed
             element divisions will correspond to the existing mesh (the
             word MESHED or EXISTING can also be substituted). Also, for
             Label = LDIV, if you execute any meshing command (such as
             AMESH or VMESH), KEY is set to 0 (MESHED) automatically. If
             Label = FBCS, then KEY= 0 indicates that boundary condition
             scaling will not be common. The applied and derived
             forces/moments will be scaled to their respective maximum
             values.

        1  - Include symbol. If Label = LDIV, then KEY = 1 indicates that the displayed line
             divisions will correspond to the value assigned by LESIZE (the
             word LESIZE can also be substituted). Also, for Label = LDIV,
             if you execute the LESIZE command, KEY is set to 1 (LESIZE)
             automatically. If Label = FBCS, then KEY= 1 indicates that
             boundary condition scaling will be common. The applied and
             derived forces/moments will be scaled to the same maximum
             value.

        N  - If Label = LAYR, then N is equal to the layer number. If Label = DOT, then N
             can be equal to 0,1,.....15, indicating the dot size. If Label
             = LDIV, then KEY = -1, indicates that no element divisions
             will be displayed (the word OFF can also be substituted).

    Notes
    -----
    Includes various symbols on the display.  Triads are right-handed with
    x displayed as the longest leg.  Where color is displayed, x is white,
    y is green, and z is blue.  For beams, x is always along the length of
    the element.   For lines, an arrow represents the direction of a line,
    from the beginning keypoint to the end keypoint.  See /PLOPTS command
    for additional display options.  Use /PSTATUS or /PSYMB,STAT to display
    settings.  Use /PSYMB,DEFA to reset all specifications back to their
    defaults.  The command /PSYMB,ECON,1 causes the symbol "M" to be
    displayed on keypoints and lines associated with meshed entities.  When
    you issue the command /PSYMB,DOT,1, a larger symbol is displayed for
    each node and keypoint location.

    PowerGraphics (/GRAPHICS,POWER) does not support /PSYMB,ESYS and
    /PSYMB,LAYR.

    If KEY = N and PowerGraphics is off, the centroid of the surface
    elements is connected to the extra node using a gray line.  However, if
    PowerGraphics is on, the color of the line connecting the centroid to
    the extra node is the same as that for the elements themselves (as
    determined by /PNUM).

    When Label = LAYR, the layer systems can be visualized with all
    current-technology layered elements and the smeared reinforcing element
    REINF265. To use /PSYMB,LAYR with REINF265, first set the vector-mode
    graphics option (/DEVICE,VECTOR,1).

    This command is valid in any processor.
    """
    command = f"/PSYMB,{label},{key}"
    return self.run(command, **kwargs)


def triad(self, lab="", **kwargs):
    """Shows the global XYZ coordinate triad on displays.

    APDL Command: /TRIAD

    Parameters
    ----------
    lab
        Display triad as follows:

        ORIG - Display triad at global origin (default).

        OFF - Turn off triad display.

        LBOT - Display triad in lower left screen corner.

        RBOT - Display triad in lower right screen corner.

        LTOP - Display triad in upper left screen corner.

        RTOP - Display triad in upper right screen corner.

    Notes
    -----
    For efficiency, ANSYS 3-D graphics logic maintains a single data
    structure (segment), which includes the triad as a 3-D data object.  If
    a 3-D device is involved (/SHOW,3D), and the ANSYS graphics are not
    being displayed as multi-plots, then the triad location is determined
    by the view settings for Window #1.  A request for triad display
    anywhere except for the origin could yield an improper display in
    windows 2 through 5. The program displays the same segment in all
    windows.  The view settings of each window constitute the only
    difference in the display in the active windows.

    This command is valid in any processor.
    """
    command = f"/TRIAD,{lab}"
    return self.run(command, **kwargs)


def udoc(self, wind="", cl_ass="", key="", **kwargs):
    """Determines position and content for the multi-legend options.

    APDL Command: /UDOC

    Parameters
    ----------
    wind
        The window number to which the command applies. (defaults to 1)

    class
        The type (and relative importance) of legend item being displayed:

        CNTR - Contour legend. This legend item is controlled separately from the other legend
               items (see note below).

        DATE - The items in the DATE class include the date and time, or the ANSYS graphical
               logo (/PLOPTS,LOGO,1). This item is shown by default in all
               plots.

        GWIN - The items in the GWIN class include the entity acronyms that appear in the
               legend of a multiplot of entities (Nodes, Elements,
               Keypoints, Lines, Areas, Volumes). GWIN items are shown by
               default for all GPLOT displays.

        TYPE - Items in the TYPE class include the plot type (e.g. ELEMENTS, MATERIALS, NODAL
               SOLUTIONS, etc.). TYPE items are shown by default in all
               plots.

        TYP2 - Items in the TYP2 class include supplementary type information, such as DMAX
               and SMAX for nodal solutions. TYP2 items are shown by
               default in all plots.

        INUM - Items in the INUM class include the number labels generated by the /PNUM
               command. This class is displayed by default in all plots
               that contain /PNUM information.

        BCDC - The items in the BCDC class include labels created by the /PBC  command. This
               class is shown by default in all plots which contain /PBC
               information.

        VECT - Items in the VECT class include labels created by the PLVECT command. This
               class is shown by default for all PLVECT plots.

        SURF - The items in the SURF class include labels from the /PSF legend. This class is
               shown by default on all plots of surface boundary
               conditions.

        BODY - Items from the BODY class include labels from the /PBF legend. This class is
               shown by default in all plots of body forces.

        PSTA - Items from the PSTA class include stress scaling statistics, such as the
               /SSCALE setting. This class is not shown as the default for
               any type of plot, and must be specifically referenced to
               display the included data.

        VIEW - The items in the VIEW class include view statistics. This class is not shown as
               the default for any type of plot, and must be specifically
               referenced to display the included data.

        MISC - The items in the MISC class include supplementary labels like /EXPANDED and
               Stress Section Cross Section. This class is not shown as the
               default for any type of plot, and must be specifically
               referenced to display the included data.

    key
        Switch:

    Notes
    -----
    The legend classes conform to the controls specified in the window
    options panel (PlotCtrls> Window Controls> Window Options). In many
    instances, the legend controls specified with the /PLOPTS command will
    take precedence and override /UDOC specifications. For instance:

    /PLOPTS,LEG1,OFF will disable the TYPE, TYP2, INUM, and MISC classes,
    regardless of the /UDOC settings.

    /PLOPTS,LEG2,OFF will disable the VIEW class, regardless of the /UDOC
    settings.

    /PLOPTS,LEG3,OFF will disable the PSTA class, regardless of the /UDOC
    settings.

    All items in a class are listed with the same X coordinate (except for
    contours). The contents of the text classes are dumped onto the display
    window from top to bottom, in order of class importance.

    The font specification for text items that are included in the user-
    specified legends are controlled with the /DEVICE command (PlotCtrls>
    Font Controls> Anno/Graph Font).

    The floating point values for the data presented in the legend(s) are
    controlled by the /GFORMAT command.
    """
    command = f"/UDOC,{wind},{cl_ass},{key}"
    return self.run(command, **kwargs)
